/* Projekt      : YTREE
 * Autor        : W. Bregulla
 * Modul        : dirwin.c
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : Funktionen zur Handhabung des DIR-Windows
 * History
 */

#include "ytree.h"


#define MIN( a, b ) ( ( (a) < (b) ) ? (a) : (b) )
#define MAX( a, b ) ( ( (a) > (b) ) ? (a) : (b) )


extern void exit(int);


static DirEntryList *dir_entry_list;
static int current_dir_entry;
static int total_dirs;

static void ReadDirList(DirEntry *dir_entry);
static void PrintDirEntry(int entry_no, int y, unsigned char hilight);
static void DisplayTree(int start_entry_no, int hilight_no);
static void BuildDirEntryList(DirEntry *dir_entry);




static void BuildDirEntryList(DirEntry *dir_entry)
{
  if( dir_entry_list ) 
  {
    free( dir_entry_list );
    dir_entry_list = NULL;
  }

  /* fuer !ANSI-Systeme.. */
  /*----------------------*/
  
  if( statistic.disk_total_directories == 0 )
  {
    dir_entry_list = NULL;
  }
  else
  {
    if( ( dir_entry_list = ( DirEntryList *) 
		             calloc( statistic.disk_total_directories,
	        	     sizeof( DirEntryList )
		           ) ) == NULL )
    {
      ERROR( "Calloc Failed*ABORT" );
      exit( 1 );
    }
  }

  current_dir_entry = 0;

  ReadDirList( dir_entry );

  total_dirs = current_dir_entry;

}




static void ReadDirList(DirEntry *dir_entry)
{
  DirEntry    *de_ptr;
  static int  level = 0;
  static unsigned long indent = 0L;

  for( de_ptr = dir_entry; de_ptr; de_ptr = de_ptr->next )
  {
    indent &= ~( 1L << level );
    if( de_ptr->next ) indent |= ( 1L << level );

    dir_entry_list[current_dir_entry].dir_entry = de_ptr;
    dir_entry_list[current_dir_entry].level = (unsigned short) level;
    dir_entry_list[current_dir_entry].indent = indent;

    current_dir_entry++;
   
    if( de_ptr->sub_tree ) 
    {
      level++;
      ReadDirList( de_ptr->sub_tree );
      level--;
    }
  }
}





static void PrintDirEntry(int entry_no, int y, unsigned char hilight)
{
  unsigned int j;
  int n;
  char buffer[32*3+PATH_LENGTH+1];
  char *dir_name;

  buffer[0] = '\0';

  for(j=0; j < dir_entry_list[entry_no].level; j++) 
  {
    if( dir_entry_list[entry_no].indent & ( 1L << j ) )
      (void) strcat( buffer, "|  " );
    else
      (void) strcat( buffer, "   " );
  }

  if( dir_entry_list[entry_no].dir_entry->next )
    (void) strcat( buffer, "6--" );
  else
    (void) strcat( buffer, "3--" );
 
  PrintSpecialString( dir_window, y, 0, buffer );
  n = (int) strlen( buffer );

  if( hilight ) wattrset( dir_window, A_REVERSE );
  
  dir_name = dir_entry_list[entry_no].dir_entry->name;
  (void) strcpy( buffer, ( *dir_name ) ? dir_name : "." );
  
  /* Scrollbares Dir-Window noch nicht implementiert... Sorry */
  /*----------------------------------------------------------*/
  
  n = DIR_WINDOW_WIDTH - n - 1;
  buffer[(n < 0) ? 0 : n] = '\0';

#ifdef NO_HIGHLIGHT
  (void) strcat( buffer, (hilight) ? " <" : "  " ); 
#endif /* NO_HIGHLIGHT */

  waddstr( dir_window, buffer );
  if( hilight ) wattrset( dir_window, 0 );
}





static void DisplayTree(int start_entry_no, int hilight_no)
{
  int i, y;
 
  y = -1;
  werase( dir_window );

  for(i=0; i < DIR_WINDOW_HEIGHT; i++)
  {
    if( start_entry_no + i >= statistic.disk_total_directories ) break;
 
    if( start_entry_no + i != hilight_no )
      PrintDirEntry( start_entry_no + i, i, FALSE );
    else
      y = i;
  }

  if( y >= 0 ) PrintDirEntry( start_entry_no + y, y, TRUE );
  
}



int HandleDirWindow(DirEntry *start_dir_entry,
		    int disp_begin_pos,
		    int cursor_pos
		   )
{
  DirEntry  *dir_entry;
  FileEntry *fe_ptr;
  int i, ch, unput_char;
  BOOL need_dsp_help;
  char new_name[PATH_LENGTH + 1];
  char new_login_path[PATH_LENGTH + 1];

  /* Merker loeschen */
  /*-----------------*/

  statistic.disp_begin_pos = 0;
  statistic.cursor_pos     = 0;


  need_dsp_help = TRUE;

  BuildDirEntryList( start_dir_entry );
  
  dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
 
  DisplayDiskStatistic();

  DisplayFileWindow( dir_entry, 0, -1 );
  wnoutrefresh( file_window );
  DisplayTree( disp_begin_pos, disp_begin_pos + cursor_pos );

  if( dir_entry->login_flag )
  {
    dir_entry->login_flag = FALSE;
    if( dir_entry->global_flag )
    {
      unput_char = 'S';
    }
    else
    {
      unput_char = CR;
    }
  }

  do
  {
    if( need_dsp_help )
    {
      need_dsp_help = FALSE;
      DisplayDirHelp();
    }

    DisplayDirParameter( dir_entry );
    wnoutrefresh( dir_window );

    if( unput_char )
    {
      ch = unput_char;
      unput_char = '\0';
    }
    else
    {
      doupdate();
      ch = getch();
    }

#ifdef VI_KEYS
    
    ch = ViKey( ch );

#endif /* VI_KEYS */

    switch( ch )
    {
      case ' ':      break;  /* Quick-Key */

      case '\t':
      case KEY_RIGHT:
      case KEY_DOWN: if( disp_begin_pos + cursor_pos + 1 >= total_dirs )
		     {
		       /* Element nicht vorhanden */
		       /*-------------------------*/

		       beep();
                     }
		     else
		     {
		       /* Element vorhanden */
	      	       /*-------------------*/

		       if( cursor_pos + 1 < DIR_WINDOW_HEIGHT )
		       {
			 /* Element ist ohne scrollen erreichbar */
			 /*--------------------------------------*/

			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					FALSE
				      );
	
			 cursor_pos++;

			 PrintDirEntry( disp_begin_pos + cursor_pos,
					cursor_pos,
					TRUE
				      );

                       }
		       else
		       {
			 /* Es muss gescrollt werden */
			 /*--------------------------*/

			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					FALSE
				      );
	
			 scroll( dir_window );
			 disp_begin_pos++;

			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					TRUE
				      );
                       }
		       dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
                       DisplayFileWindow( dir_entry, 0, -1 );
                       wnoutrefresh( file_window );
		     }
                     break;

      case KEY_LEFT:
      case KEY_BTAB:
      case KEY_UP  : if( disp_begin_pos + cursor_pos - 1 < 0 )
		     {
		       /* Element nicht vorhanden */
		       /*-------------------------*/

		       beep();
                     }
		     else
		     {
		       /* Element vorhanden */
	      	       /*-------------------*/

		       if( cursor_pos - 1 >= 0 )
		       {
			 /* Element ist ohne scrollen erreichbar */
			 /*--------------------------------------*/

			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					FALSE
				      );
	
			 cursor_pos--;

			 PrintDirEntry( disp_begin_pos + cursor_pos,
					cursor_pos,
					TRUE
				      );

                       }
		       else
		       {
			 /* Es muss gescrollt werden */
			 /*--------------------------*/

			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					FALSE
				      );
	
			 wmove( dir_window, 0, 0 );
			 winsertln( dir_window );
			 disp_begin_pos--;

			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					TRUE
				      );
                       }
		       dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;

                       DisplayFileWindow( dir_entry, 0, -1 );
                       wnoutrefresh( file_window );
		     }
                     break;

      case KEY_NPAGE:
		     if( disp_begin_pos + cursor_pos >= total_dirs - 1 )
		     {
		       /* Letzte Position */
		       /*-----------------*/

		       beep();
                     }
		     else
		     {
		       if( cursor_pos < DIR_WINDOW_HEIGHT - 1 )
		       {
			 /* Cursor steht nicht auf letztem Eintrag
			  * der Seite
			  */
			
			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					FALSE
				      );
	
		         if( disp_begin_pos + DIR_WINDOW_HEIGHT > total_dirs  - 1 )
			   cursor_pos = total_dirs - disp_begin_pos - 1;
			 else
			   cursor_pos = DIR_WINDOW_HEIGHT - 1;

		         dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;

                         DisplayFileWindow( dir_entry, 0, -1 );
                         wnoutrefresh( file_window );

			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					TRUE
				      );
		       }
		       else
		       {
			 /* Scrollen */
			 /*----------*/

			 if( disp_begin_pos + cursor_pos + DIR_WINDOW_HEIGHT < total_dirs )
			 {
			   disp_begin_pos += DIR_WINDOW_HEIGHT;
			   cursor_pos = DIR_WINDOW_HEIGHT - 1;
			 }
			 else
			 {
			   disp_begin_pos = total_dirs - DIR_WINDOW_HEIGHT;
			   if( disp_begin_pos < 0 ) disp_begin_pos = 0;
			   cursor_pos = total_dirs - disp_begin_pos - 1;
			 }

		         dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;

                         DisplayFileWindow( dir_entry, 0, -1 );
                         wnoutrefresh( file_window );
                         DisplayTree( disp_begin_pos, 
				      disp_begin_pos + cursor_pos 
				    );
		       } 
		     }
                     break;


      case KEY_PPAGE:
		     if( disp_begin_pos + cursor_pos <= 0 )
		     {
		       /* Erste Position */
		       /*----------------*/

		       beep();
                     }
		     else
		     {
		       if( cursor_pos > 0 )
		       {
			 /* Cursor steht nicht auf erstem Eintrag
			  * der Seite
			  */
			
			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					FALSE
				      );
	
			 cursor_pos = 0;

		         dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;

                         DisplayFileWindow( dir_entry, 0, -1 );
                         wnoutrefresh( file_window );

			 PrintDirEntry( disp_begin_pos + cursor_pos, 
					cursor_pos,
					TRUE
				      );
		       }
		       else
		       {
			 /* Scrollen */
			 /*----------*/

			 if( (disp_begin_pos -= DIR_WINDOW_HEIGHT) < 0 )
			 {
			   disp_begin_pos = 0;
			 }

                         cursor_pos = 0;

		         dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;

                         DisplayFileWindow( dir_entry, 0, -1 );
                         wnoutrefresh( file_window );
                         DisplayTree( disp_begin_pos, 
				      disp_begin_pos + cursor_pos 
				    );
		       } 
		     }
                     break;


      case KEY_HOME: if( disp_begin_pos == 0 && cursor_pos == 0 )
		     { 
		       /* Position 1 bereits errecht */
		       /*----------------------------*/

		       beep();
		     }
		     else
		     {
		       disp_begin_pos = 0;
		       cursor_pos     = 0;
		       
		       dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;

                       DisplayFileWindow( dir_entry, 0, -1 );
                       wnoutrefresh( file_window );
                       DisplayTree( disp_begin_pos, 
				    disp_begin_pos + cursor_pos 
				  );
		       
		     }
                     break;


      case KEY_END : disp_begin_pos = MAX(0, total_dirs - DIR_WINDOW_HEIGHT);
		     cursor_pos     = total_dirs - disp_begin_pos - 1;
		       
		     dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;

                     DisplayFileWindow( dir_entry, 0, -1 );
                     wnoutrefresh( file_window );
                     DisplayTree( disp_begin_pos, 
				  disp_begin_pos + cursor_pos 
				);
                     break;

      case 'F':
      case 'f':      ReadFileSpec();
		     
                     DisplayFileWindow( dir_entry, 0, -1 );
                     wnoutrefresh( file_window );
		     DisplayDiskStatistic();
		     need_dsp_help = TRUE;
                     break;

      case 'T' :
      case 't' :     for(fe_ptr=dir_entry->file; fe_ptr; fe_ptr=fe_ptr->next)
		     {
		       if( fe_ptr->matching ) 
		       {
			 if( !fe_ptr->tagged )
			 {
			   fe_ptr->tagged = TRUE;

	       	           dir_entry->tagged_files++;
		           dir_entry->tagged_bytes += fe_ptr->stat_struct.st_size;
	       	           statistic.disk_tagged_files++;
		           statistic.disk_tagged_bytes += fe_ptr->stat_struct.st_size;
			 }
		       }
		     }
                     DisplayFileWindow( dir_entry, 0, -1 );
                     wnoutrefresh( file_window );
		     DisplayDiskStatistic();
		     break;

      case 'U' :
      case 'u' :     for(fe_ptr=dir_entry->file; fe_ptr; fe_ptr=fe_ptr->next)
		     {
		       if( fe_ptr->matching ) 
		       {
			 if( fe_ptr->tagged )
			 {
			   fe_ptr->tagged = FALSE;

	       	           dir_entry->tagged_files--;
		           dir_entry->tagged_bytes -= fe_ptr->stat_struct.st_size;
	       	           statistic.disk_tagged_files--;
		           statistic.disk_tagged_bytes -= fe_ptr->stat_struct.st_size;
			 }
		       }
		     }
                     DisplayFileWindow( dir_entry, 0, -1 );
                     wnoutrefresh( file_window );
		     DisplayDiskStatistic();
		     break;

      case 'T' & 0x1F :
		     for(i=0; i < total_dirs; i++)
		     {
                       for(fe_ptr=dir_entry_list[i].dir_entry->file; 
			   fe_ptr; 
			   fe_ptr=fe_ptr->next
			  )
		       {
		         if( fe_ptr->matching ) 
		         {
			   if( !fe_ptr->tagged )
			   {
			     fe_ptr->tagged = TRUE;

	       	             dir_entry->tagged_files++;
		             dir_entry->tagged_bytes += fe_ptr->stat_struct.st_size;
	       	             statistic.disk_tagged_files++;
		             statistic.disk_tagged_bytes += fe_ptr->stat_struct.st_size;
			   }
			 }
		       }
		     }
                     DisplayFileWindow( dir_entry, 0, -1 );
                     wnoutrefresh( file_window );
		     DisplayDiskStatistic();
		     break;

      case 'U' & 0x1F :
		     for(i=0; i < total_dirs; i++)
		     {
                       for(fe_ptr=dir_entry_list[i].dir_entry->file; 
			   fe_ptr; 
			   fe_ptr=fe_ptr->next
			  )
		       {
		         if( fe_ptr->matching ) 
		         {
			   if( fe_ptr->tagged )
			   {
			     fe_ptr->tagged = FALSE;

	       	             dir_entry->tagged_files--;
		             dir_entry->tagged_bytes -= fe_ptr->stat_struct.st_size;
	       	             statistic.disk_tagged_files--;
		             statistic.disk_tagged_bytes -= fe_ptr->stat_struct.st_size;
			   }
			 }
		       }
		     }
                     DisplayFileWindow( dir_entry, 0, -1 );
                     wnoutrefresh( file_window );
		     DisplayDiskStatistic();
		     break;

      
      case 'F' & 0x1F :  
		     RotateFileMode();
                     DisplayFileWindow( dir_entry, 0, -1 );
                     wnoutrefresh( file_window );
		     DisplayDiskStatistic();
		     need_dsp_help = TRUE;
		     break;

      case 'S':
      case 's':      if( statistic.disk_matching_files )
		     {
		       /* Cursor-Position sichern */
		       /*-------------------------*/

		       statistic.disp_begin_pos = disp_begin_pos;
		       statistic.cursor_pos     = cursor_pos;

		       if( HandleFileWindow( dir_entry, 
					     dir_entry->start_file,
					     dir_entry->cursor_pos,
					     TRUE ) != LOGIN_ESC )
		       {
		         DisplayDiskStatistic();
                         DisplayFileWindow( dir_entry, 0, -1 );
                         wnoutrefresh( file_window );
                         DisplayTree( disp_begin_pos, 
				      disp_begin_pos + cursor_pos 
				    );
		         
			 /* Cursor-Position zuruecksetzen */
		         /*-------------------------------*/

		         statistic.disp_begin_pos = 0;
		         statistic.cursor_pos     = 0;

		       }
		       else
		       {
			 /* wie bei Login */
			 /*---------------*/

			 ch = 'L';
		       }
		     }
		     else
		     {
		       beep();
		     }
		     need_dsp_help = TRUE;
		     break;
   
      case LF :
      case CR :      
		    if( dir_entry->matching_files )
		     {
		       /* Cursor-Position sichern */
		       /*-------------------------*/

		       statistic.disp_begin_pos = disp_begin_pos;
		       statistic.cursor_pos     = cursor_pos;

		       if( HandleFileWindow( dir_entry, 
					     dir_entry->start_file,
					     dir_entry->cursor_pos,
					     FALSE ) != LOGIN_ESC )
                       {
			 DisplayFileWindow( dir_entry, 0, -1 );
                         wnoutrefresh( file_window );
                         DisplayTree( disp_begin_pos, 
				      disp_begin_pos + cursor_pos 
				    );
		         
			 DisplayDiskStatistic();
			 
			 /* Cursor-Position zuruecksetzen */
		         /*-------------------------------*/

		         statistic.disp_begin_pos = 0;
		         statistic.cursor_pos     = 0;

		       }
		       else
		       {
			 /* wie bei Login */
			 /*---------------*/

			 ch = 'L';
		       }
		       need_dsp_help = TRUE;
		     }
		     else
		     {
		       beep();
		     }
		     break;
      case 'X':
      case 'x':      (void) Execute( dir_entry, NULL );
		     need_dsp_help = TRUE;
		     DisplayAvailBytes();
		     break;

      case 'M':      
      case 'm':      if( !MakeDirectory( dir_entry ) )
		     {
		       BuildDirEntryList( start_dir_entry );
                       DisplayTree( disp_begin_pos, 
				    disp_begin_pos + cursor_pos 
				  );
		       DisplayAvailBytes();
		     } 
		     need_dsp_help = TRUE;
		     break;
      case 'D':
      case 'd':      if( !DeleteDirectory( dir_entry ) )
		     {
		       BuildDirEntryList( start_dir_entry );
		       
		       if( disp_begin_pos + cursor_pos > 0 )
		       {
			 if( cursor_pos > 0 ) cursor_pos--;
			 else disp_begin_pos--;
		       }

		       dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
                       DisplayFileWindow( dir_entry, 0, -1 );
                       wnoutrefresh( file_window );
                       
		       DisplayTree( disp_begin_pos, 
				    disp_begin_pos + cursor_pos 
				  );
		       DisplayAvailBytes();
		     } 
		     need_dsp_help = TRUE;
		     break;
      
      case 'r':
      case 'R':      if( !GetRenameParameter( dir_entry->name, new_name ) )
                     {
		       if( !RenameDirectory( dir_entry, new_name ) )
		       {
			 /* Rename OK */
			 /*-----------*/
			 
		         BuildDirEntryList( start_dir_entry );
                         DisplayTree( disp_begin_pos, 
				      disp_begin_pos + cursor_pos 
				    );
		         DisplayAvailBytes();
		         dir_entry = dir_entry_list[disp_begin_pos + cursor_pos].dir_entry;
                       }
		     }

		     need_dsp_help = TRUE;
		     break;

/*       case 'G':  */
      case 'g':      (void) ChangeDirGroup( dir_entry );
		     need_dsp_help = TRUE;
		     break;

      case 'O':
      case 'o':      (void) ChangeDirOwner( dir_entry );
		     need_dsp_help = TRUE;
		     break;

      case 'A':
      case 'a':      (void) ChangeDirModus( dir_entry );
		     need_dsp_help = TRUE;
		     break;

      case 'Q':
      case 'q':      break;

#ifndef VI_KEYS
      case 'l':
#endif /* VI_KEYS */
      case 'L':      if( mode != DISK_MODE ) 
		       (void) strcpy( new_login_path, disk_statistic.login_path );
		     else
		       (void) GetPath( dir_entry, new_login_path );
		     if( !GetNewLoginPath( new_login_path ) )
		     {
		       DisplayMenu();
		       DisplayVersion();
		       doupdate();
		       
		       /* Cursor-Position sichern */
		       /*-------------------------*/

		       statistic.disp_begin_pos = disp_begin_pos;
		       statistic.cursor_pos     = cursor_pos;

		       (void) LoginDisk( new_login_path );
		     }
		     need_dsp_help = TRUE;
		     break;

      case 'L' & 0x1F:
		     clearok( stdscr, TRUE );
		     break;

      default :      beep();
		     break;
    } /* switch */
  } while( (ch != 'q') && (ch != 'Q') && (ch != 'l') && (ch != 'L') );

  return( ch );
}



