/* Projekt      : YTREE
 * Autor        : W. Bregulla
 * Modul        : disp.c
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : Menue-Geruest anzeigen
 * History
 */

#include "ytree.h"
#include "patchlev.h"


extern void exit(int);


static void PrintMenuLine(WINDOW *win, int y, int x, char *line);


static char *mask[] = {
" Path:",
"1-5--------------------2",
"| |FILE:               |",
"| 6--------------------7",
"| |DISK:               |",
"| | Available          |",
"| |  Bytes:            |",
"| 6--------------------7",
"| |DISK Statistics     |",
"| | Total              |",
"| |  Files:            |",
"| |  Bytes:            |",
"| | Matching           |",
"| |  Files:            |",
"| |  Bytes:            |",
"| | Tagged             |",
"6-7  Files:            |",
"| |  Bytes:            |",
"| | Current Directory  |",
"| |                    |",
"| |  Bytes:            |"
};


static char *logo[] = {
                        "                  #                       ",
                        "                ##                        ",
                        "     ##  ##   #####  ## ###   ####    ####",
                        "    ##  ##    ##     ##  ## ##  ##  ##  ##",
                        "   ##  ##    ##     ##  ## ######  ###### ",
                        "   #####    ## #   ##     ##      ##      ",
                        "     ##     ###  ####     ####    ####    ",
                        "#####                                     "
		      };

static char *extended_line = "| |                    |";


static char *last_line = "3-8--------------------4";


static char *dir_help[MAX_MODES][2] = 
  {
    { /* DISK_MODE */
      "DIR      Attribute Delete  Filespec  Group Log Makedir Quit                ",
      "COMMANDS Owner Rename Showall Tag Untag eXecute ^Filemode                  "
    },
    { /* LL_FILE_MODE */
      "DIR      Filespec Login Showall Tag Untag eXecute ^Filemode Quit           ",
      "COMMANDS                                                                   "
    },
    { /* TAR_FILE_MODE */
      "TAR-DIR  Filespec Log Showall Tag Untag eXecute ^Filemode Quit             ",
      "COMMANDS                                                                   "
    },
    { /* ZOO_FILE_MODE */
      "ZOO-DIR  Filespec Log Showall Tag Untag eXecute ^Filemode Quit             ",
      "COMMANDS                                                                   "
    },
    { /* ZIP_FILE_MODE */
      "ZIP-DIR  Filespec Log Showall Tag Untag eXecute ^Filemode Quit             ",
      "COMMANDS                                                                   "
    },
    { /* LHA_FILE_MODE */
      "LHA-DIR  Filespec Log Showall Tag Untag eXecute ^Filemode Quit             ",
      "COMMANDS                                                                   "
    },
    { /* ARC_FILE_MODE */
      "ARC-DIR  Filespec Log Showall Tag Untag eXecute ^Filemode Quit             ",
      "COMMANDS                                                                   "
    }
  };


static char *file_help[MAX_MODES][2] = 
  {
    { /* DISK_MODE */
      "FILE      Attribute Copy Delete Edit Filespec Group Hex Login Quit         ",
      "COMMANDS  Move Owner Pipe Rename Sort Tag Untag View eXecute ^Filemode     "
    },
    { /* LL_FILE_MODE */
      "FILE      Filespec Login Sort Tag Untag eXecute ^Filemode      Quit        ",
      "COMMANDS                                                                   "
    },
    { /* TAR_FILE_MODE */
      "TAR-FILE  Copy Filespec Hex Pipe Sort Tag Untag View ^Filemode Quit        ",
      "COMMANDS                                                                   "
    },
    { /* ZOO_FILE_MODE */
      "ZOO-FILE  Copy Filespec Hex Pipe Sort Tag Untag View ^Filemode Quit        ",
      "COMMANDS                                                                   "
    },
    { /* ZIP_FILE_MODE */
      "ZIP-FILE  Copy Filespec Hex Pipe Sort Tag Untag View ^Filemode Quit        ",
      "COMMANDS                                                                   "
    },
    { /* LHA_FILE_MODE */
      "LHA-FILE  Copy Filespec Hex Pipe Sort Tag Untag View ^Filemode Quit        ",
      "COMMANDS                                                                   "
    },
    { /* ARC_FILE_MODE */
      "ARC-FILE  Copy Filespec Hex Pipe Sort Tag Untag View ^Filemode Quit        ",
      "COMMANDS                                                                   "
    }
  };


static char version[80];



void DisplayVersion(void)
{
  ClearHelp();
  (void) sprintf( version, 
		  "ytree Version %sPL%d %s (WB)", 
		  VERSION, 
		  PATCHLEVEL, 
		  VERSIONDATE 
		);
  mvaddstr( LINES - 2, (unsigned) (COLS - strlen( version )) >> 1, version );
}
   
   


void DisplayDirHelp(void)
{
  int i;

  for( i=0; i < sizeof(dir_help[mode]) / sizeof(dir_help[mode][0]); i++)
    mvaddstr( LINES - 2 + i, 0, dir_help[mode][i] );
}
   


void DisplayFileHelp(void)
{
  int i;

  for( i=0; i < sizeof(file_help[mode]) / sizeof(file_help[mode][0]); i++)
    mvaddstr( LINES - 2 + i, 0, file_help[mode][i] );
}
   


void ClearHelp(void)
{
  int i;

  for( i=0; i < 2; i++ )
  {
    move( LINES - 2 + i, 0 ); clrtoeol();
  }
}
   


void DisplayMenu(void)
{
  int    y;
  int    lines, columns;
  
  
  PrintSpecialString( stdscr, 0, 0, mask[0] );
  
  for( y=1; y < sizeof(mask) / sizeof(mask[0]); y++)
    PrintMenuLine( stdscr, y, 0, mask[y] );

  for( ; y < LINES - 3; y++ )
    PrintMenuLine( stdscr, y, 0, extended_line );
    
  PrintMenuLine( stdscr, y, 0, last_line );

  lines = sizeof(logo) / sizeof(logo[0]);
  columns = strlen( logo[0] );

  for( y=0; y < lines; y++ )
  {
    mvwaddstr( dir_window, 
	       y + ((DIR_WINDOW_HEIGHT - lines) >> 1), 
	       (DIR_WINDOW_WIDTH - columns) >> 1,
	       logo[y]
	     );
  }
}



void SwitchToSmallFileWindow(void)
{ 
  werase( file_window );
  PrintMenuLine( stdscr, 16, 0, mask[16] );
  file_window = small_file_window;
  wnoutrefresh( stdscr );
}


void SwitchToBigFileWindow(void)
{
  werase( file_window );
  mvaddch( DIR_WINDOW_Y + DIR_WINDOW_HEIGHT,  
	   DIR_WINDOW_X - 1, 
	   ACS_VLINE
	 );
  mvaddch( DIR_WINDOW_Y + DIR_WINDOW_HEIGHT, 
	   DIR_WINDOW_X + DIR_WINDOW_WIDTH,
	   ACS_VLINE
	 );
  file_window = big_file_window;
  wnoutrefresh( stdscr );
}




static void PrintMenuLine(WINDOW *win, int y, int x, char *line)
{
  int  i;
  char *buffer;

  if( ( buffer = malloc( COLS + 2 ) ) == NULL )
  {
     ERROR( "Malloc failed*ABORT" );
     exit( 1 );
  }

  buffer[0] = line[0];

  for(i=1; i < (int) (COLS - strlen( line ) + 2); i++)
    buffer[i] = line[1];

  (void) strcpy( &buffer[i], &line[2] );

  PrintSpecialString( win, y, x, buffer );

  free( buffer );
}

