/* Projekt      : YTREE
 * Autor        : W. Bregulla
 * Modul        : hex.c
 * Datum        : 16.08.92
 * Version      : 1.0
 * Beschreibung : View-Hex-Kommando-Bearbeitung
 * History
 */


#include "ytree.h"


extern void exit(int);
extern char *sys_errlist[];

static int ViewHexFile(char *file_path);
static int ViewHexArchiveFile(char *file_path);




int ViewHex(char *file_path)
{
  switch( mode )
  {
    case DISK_MODE :     return( ViewHexFile( file_path ) );
    case TAR_FILE_MODE : return( ViewHexArchiveFile( file_path ) );
    case ZOO_FILE_MODE : return( ViewHexArchiveFile( file_path ) );
    case ZIP_FILE_MODE : return( ViewHexArchiveFile( file_path ) );
    case LHA_FILE_MODE : return( ViewHexArchiveFile( file_path ) );
    case ARC_FILE_MODE : return( ViewHexArchiveFile( file_path ) );
    default:             beep(); return( -1 );
  }
}




static int ViewHexFile(char *file_path)
{
  char *command_line;
  int  compress_method;
  int result = -1;

  if( access( file_path, R_OK ) )
  {
    (void) sprintf( message, 
		    "HexView not possible!*\"%s\"*%s", 
		    file_path, 
		    sys_errlist[errno] 
		  );
    MESSAGE( message );
    ESCAPE;
  }

  if( ( command_line = malloc( COMMAND_LINE_LENGTH + 1 ) ) == NULL )
  {
    ERROR( "Malloc failed*ABORT" );
    exit( 1 );
  }


  compress_method = GetCompressMethod( file_path );


  if( compress_method == FREEZE_COMPRESS )
  {
    (void) sprintf( command_line, 
		    "%s < %s | %s | %s %s", 
		    MELT, 
		    file_path, 
		    HEXDUMP, 
		    PAGER, 
		    PAGEOPT
		  );
  }
  else if( compress_method == COMPRESS_COMPRESS )
  {
    (void) sprintf( command_line, 
		    "%s < %s | %s | %s %s", 
		    UNCOMPRESS, 
		    file_path, 
		    HEXDUMP, 
		    PAGER, 
		    PAGEOPT
		  );
  }
  else if( compress_method == GZIP_COMPRESS )
  {
    (void) sprintf( command_line, 
		    "%s < %s | %s | %s %s", 
		    GNUUNZIP, 
		    file_path, 
		    HEXDUMP, 
		    PAGER, 
		    PAGEOPT
		  );
  }
  else
  {
    (void) sprintf( command_line, 
		    "%s %s | %s %s", 
		    HEXDUMP, 
		    file_path, 
		    PAGER, 
		    PAGEOPT 
		  );
  }
    
  
  result = SystemCall( command_line );

  free( command_line );

FNC_XIT:

  return( result );
}



static int ViewHexArchiveFile(char *file_path)
{
  char *command_line;
  char buffer[80];
  int result = -1;

  if( ( command_line = malloc( COMMAND_LINE_LENGTH + 1 ) ) == NULL )
  {
    ERROR( "Malloc failed*ABORT" );
    exit( 1 );
  }

  (void) sprintf( buffer, "| %s | %s %s", HEXDUMP, PAGER, PAGEOPT );

  MakeExtractCommandLine( command_line, 
			  statistic.login_path, 
			  file_path, 
			  buffer
			);

  result = SystemCall( command_line );

  free( command_line );

  return( result );
}


