/* Projekt      : YTREE
 * Autor        : W. Bregulla
 * Modul        : login.c
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : Dateibaum lesen
 * History
 */



#include "ytree.h"

extern char *sys_errlist[];

static void DeleteTree(DirEntry *tree);



static void DeleteTree(DirEntry *tree)
{
  DirEntry  *de_ptr, *next_de_ptr;
  FileEntry *fe_ptr, *next_fe_ptr;
  
  for( de_ptr=tree; de_ptr; de_ptr=next_de_ptr)
  {
    next_de_ptr = de_ptr->next;
    
    for( fe_ptr=de_ptr->file; fe_ptr; fe_ptr=next_fe_ptr)
    {
      next_fe_ptr=fe_ptr->next;
      free( fe_ptr );
    }
    
    if( de_ptr->sub_tree ) DeleteTree( de_ptr->sub_tree );

    free( de_ptr );
  }
}




/* Login Disk liefert
 * -1 bei Fehler
 * 0  bei fehlerfreiem lesen einen neuen Baumes
 * 1  bei benutzung des Baumes im Speicher
 */


int LoginDisk(char *path)
{
  struct stat stat_struct;
  char   command_line[COMMAND_LINE_LENGTH + 1];
  char   cat_file_name[PATH_LENGTH+1];
  int    compress_method;
  int    pid;
  int    p[2];
  int    l;
  FILE   *f;
  int    status;

  if( STAT( path, &stat_struct ) )
  {
    /* Stat failed */
    /*-------------*/

    (void) sprintf( message, "Can't access*\"%s\"*%s", path, sys_errlist[errno] );
    MESSAGE( message );
    return( -1 );
  }


  if( !strcmp( disk_statistic.login_path, path ) )
  {
    /* Baum ist noch im Speicher! verwenden ! */
    /*----------------------------------------*/

    if( statistic.tree != disk_statistic.tree ) DeleteTree( statistic.tree );

    mode = DISK_MODE;
    (void) memcpy( (char *) &statistic,
		   (char *) &disk_statistic,
		   sizeof( Statistic )
	         );
    (void) SetFileSpec( statistic.file_spec );
    return( 1 );   /* Return-Wert fuer "alten Baum" */
  }
   

  DisplayMenu();
  DisplayVersion();
  wnoutrefresh( stdscr );
  wnoutrefresh( dir_window );
  doupdate();

  if( mode != DISK_MODE ) DeleteTree( statistic.tree ); 
  else
  {
    /* Status retten */
    /*---------------*/
    (void) memcpy( (char *) &disk_statistic, 
		   (char *) &statistic,
		   sizeof( Statistic )
		 );
  } 
  
  (void) memset( &statistic, 0, sizeof( statistic ) );
  
  if( ( statistic.tree = (DirEntry *) malloc( sizeof( DirEntry ) + 
					      PATH_LENGTH )) == NULL )
  {
    ERROR( "Malloc failed*ABORT" );
    exit( 1 );
  }
  
  (void) memset( statistic.tree, 0, sizeof( DirEntry ) + PATH_LENGTH );

  (void) strcpy( statistic.path, path );
  (void) strcpy( statistic.login_path, path );
  (void) strcpy( statistic.file_spec, DEFAULT_FILE_SPEC );
  statistic.kind_of_sort = SORT_BY_NAME;
  (void) memcpy( &statistic.tree->stat_struct, 
		 &stat_struct, 
		 sizeof( stat_struct ) 
	       );
  
  if( !S_ISDIR(stat_struct.st_mode ) )
  {
    /* Kein Verzeichnis ==> TAR_FILE/ZOO/ZIP/LHA/ARC_FILE */
    /*----------------------------------------------------*/

    compress_method = GetCompressMethod( statistic.login_path );
    l = strlen( statistic.login_path );

    switch( compress_method )
    {
      case ZOO_COMPRESS:    mode = ZOO_FILE_MODE; break;
      case ARC_COMPRESS:    mode = ARC_FILE_MODE; break;
      case LHA_COMPRESS:    mode = LHA_FILE_MODE; break;
      case ZIP_COMPRESS:    mode = ZIP_FILE_MODE; break;
      default:              mode = TAR_FILE_MODE;
    }

    (void) strcpy( statistic.tree->name, path );
   
    if( pipe( p ) )
    {
      ERROR( "pipe failed" );
      return( -1 );
    }

    pid = fork();

    if( pid == -1 )
    {
      ERROR( "can't fork()" );
      (void) close( p[0] );
      (void) close( p[1] );
      return( -1 );
    }
    else if( pid == 0 )
    {
      /* Sohn */
      /*------*/
 
      (void) close( p[0] );
      (void) close( 1 );
      (void) dup( p[1] );
      (void) close( p[1] );

      if( compress_method == ZOO_COMPRESS )
      {
        /* zoo vom ZOO_FILE */
        /*------------------*/
    
        (void) sprintf( command_line, "%s %s", 
		        ZOOLIST, 
		        statistic.login_path
		      );
      }
      else if( compress_method == LHA_COMPRESS )
      {
        /* LHA_FILE */
        /*----------*/
    
        (void) sprintf( command_line, "%s %s", 
		        LHALIST, 
		        statistic.login_path
		      );
      }
      else if( compress_method == ZIP_COMPRESS )
      {
        /* ZIP_FILE */
        /*----------*/
    
        (void) sprintf( command_line, "%s %s", 
		        ZIPLIST, 
		        statistic.login_path
		      );
      }
      else if( compress_method == ARC_COMPRESS )
      {
        /* ARC_FILE */
        /*----------*/
    
        (void) sprintf( command_line, "%s %s", 
		        ARCLIST, 
		        statistic.login_path
		      );
      }
      else if( compress_method == FREEZE_COMPRESS )
      {
        /* melt < TAR_FILE | gtar tvf - */
        /*------------------------------*/
    
        (void) sprintf( command_line, "%s < %s | %s", 
		        MELT, 
		        statistic.login_path,
		        TARLIST
		      );
      }
      else if( compress_method == MULTIPLE_FREEZE_COMPRESS )
      {
	(void) strncpy( cat_file_name, statistic.login_path, l - 2 );
	(void) strcpy( &cat_file_name[l - 2], "*" );

        /* cat TAR_FILE | melt | gtar tvf - */
        /*----------------------------------*/
    
        (void) sprintf( command_line, "%s %s | %s | %s", 
			CAT,
		        cat_file_name,
		        MELT, 
		        TARLIST
		      );
      }
      else if( compress_method == COMPRESS_COMPRESS )
      {
        /* uncompress < TAR_FILE | gtar tvf - */
        /*------------------------------------*/
    
        (void) sprintf( command_line, "%s < %s | %s", 
		        UNCOMPRESS, 
		        statistic.login_path,
		        TARLIST
		      );
      }
      else if( compress_method == MULTIPLE_COMPRESS_COMPRESS )
      {
	(void) strncpy( cat_file_name, statistic.login_path, l - 2 );
	(void) strcpy( &cat_file_name[l - 2], "*" );
        
	/* cat TAR_FILE.X* | uncompress | gtar tvf - */
        /*-------------------------------------------*/
    
        (void) sprintf( command_line, "%s %s | %s | %s", 
			CAT,
		        cat_file_name,
		        UNCOMPRESS, 
		        TARLIST
		      );
      }
      else if( compress_method == GZIP_COMPRESS )
      {
        /* gunzip < TAR_FILE | gtar tvf - */
        /*--------------------------------*/
    
        (void) sprintf( command_line, "%s < %s | %s", 
		        GNUUNZIP, 
		        statistic.login_path,
		        TARLIST
		      );
      }
      else if( compress_method == MULTIPLE_GZIP_COMPRESS )
      {
	(void) strncpy( cat_file_name, statistic.login_path, l - 2 );
	(void) strcpy( &cat_file_name[l - 2], "*" );
        
	/* cat TAR_FILE.X* | gunzip | gtar tvf - */
        /*---------------------------------------*/
    
        (void) sprintf( command_line, "%s %s | %s | %s", 
			CAT,
		        cat_file_name,
		        GNUUNZIP, 
		        TARLIST
		      );
      }
      else
      {
	/* NO_COMPRESS */
	/*-------------*/

        /* gtar tvf - < TAR_FILE */
        /*-----------------------*/
    
        (void) sprintf( command_line, "%s < %s", 
		        TARLIST,
		        statistic.login_path
		      );
      }

#ifdef DEBUG
  fprintf( stderr, "system( \"%s\" )\n", command_line );
#endif

      if( SilentSystemCall( command_line ) )
      {
	(void) fprintf( stderr, "system(%s) failed\n", command_line );
      }
      exit( 0 );
    }
    else
    {
      /* Vater */
      /*-------*/

      (void) close( p[1] );

      if( ( f = fdopen( p[0], "r" ) ) == NULL )
      {
	ERROR( "fdopen failed" );
	return( -1 );
      }
      
      if( mode == ZOO_FILE_MODE )
      {
	if( ReadTreeFromZOO( statistic.tree, f ) )
        {
	  ERROR( "ReadTreeFromZOO Failed" );
          (void) fclose( f );
	  (void) wait( &status );
          return( -1 );
	}
      }
      else if( mode == LHA_FILE_MODE )
      {
	if( ReadTreeFromLHA( statistic.tree, f ) )
        {
	  ERROR( "ReadTreeFromLHA Failed" );
          (void) fclose( f );
	  (void) wait( &status );
          return( -1 );
	}
      }
      else if( mode == ZIP_FILE_MODE )
      {
	if( ReadTreeFromZIP( statistic.tree, f ) )
        {
	  ERROR( "ReadTreeFromZIP Failed" );
          (void) fclose( f );
	  (void) wait( &status );
          return( -1 );
	}
      }
      else if( mode == ARC_FILE_MODE )
      {
	if( ReadTreeFromARC( statistic.tree, f ) )
        {
	  ERROR( "ReadTreeFromARC Failed" );
          (void) fclose( f );
	  (void) wait( &status );
          return( -1 );
	}
      }
      else
      {
        if( ReadTreeFromTAR( statistic.tree, f ) )
        {
          ERROR( "ReadTreeFromTAR Failed" );
          (void) fclose( f );
	  (void) wait( &status );
          return( -1 );
        }
      }
      (void) wait( &status );
      (void) fclose( f );
    }
  }
  else
  {
    mode = DISK_MODE;
    if( *disk_statistic.login_path )
    {
      /* Alten Baum loeschen */
      /*---------------------*/
      *disk_statistic.login_path = '\0';
      DeleteTree( disk_statistic.tree );
    }

    (void) strcpy( statistic.tree->name, path );
  
    if( ReadTree( statistic.tree, path ) )
    {
      ERROR( "ReadTree Failed" );
      return( -1 );
    }
    (void) memcpy( (char *) &disk_statistic, 
		   (char *) &statistic,
		   sizeof( Statistic )
		 );
  } 
    
  (void) GetDiskParameter( path, statistic.disk_name, &statistic.disk_space );
  (void) SetFileSpec( statistic.file_spec );
/*  SetKindOfSort( statistic.kind_of_sort ); */
  
  return( 0 );
}





int GetNewLoginPath(char *path)
{
  int result;
  char *cptr;

  result = -1;

  ClearHelp();

  mvaddstr( LINES - 2, 1, "NEW LOGIN-PATH:" );

  if( mode == LL_FILE_MODE && *path == '<' )
  {
    for( cptr = path; *cptr = *(cptr + 1); cptr++ ) 
      ;
    if( path[strlen(path) - 1] == '>' ) path[strlen(path) - 1 ] = '\0';
  }

  if( InputString( path, LINES - 2, 17, COLS - 24, "\r\033" ) == CR )
    result = 0;

  return( result );
}


