/* Projekt      : YTREE
 * Autor        : W. Bregulla
 * Modul        : mkdir.c
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : Erstellen von Verzeichnissen
 * History
 */


#include "ytree.h"


extern char *sys_errlist[];




int MakeDirectory(DirEntry *father_dir_entry)
{
  DirEntry *den_ptr, *des_ptr;
  char buffer[PATH_LENGTH+1];
  char dir_name[PATH_LENGTH+1];
  struct stat stat_struct;
  int result = -1;

  if( mode != DISK_MODE )
  {
    beep();
    return( result );
  }

  ClearHelp();

  mvaddstr( LINES - 2, 1, "Make Subdirectory:" );

  *dir_name = '\0';

  if( InputString( dir_name, LINES - 2, 20, COLS - 20 - 1, "\r\033" ) == CR )
  {
    (void) GetPath( father_dir_entry, buffer );
    (void) strcat( buffer, FILE_SEPARATOR_STRING );
    (void) strcat( buffer, dir_name );

    if( ( result = mkdir( buffer, S_IREAD  |
		                  S_IWRITE |
		                  S_IEXEC  |
		                  S_IRGRP  |
		                  S_IWGRP  |
		                  S_IXGRP  |
		                  S_IROTH  |
		                  S_IWOTH  |
		                  S_IXOTH
      ) ) )
    {
      (void) sprintf( message, "Can't create Directory*\"%s\"*%s", 
		      buffer, sys_errlist[errno]
		    );
      MESSAGE( message );
    }
    else
    {
      /* Directory erstellt 
       * ==> einklinken im Baum 
       */
      
      if( ( den_ptr = (DirEntry *) malloc( sizeof( DirEntry ) + strlen( dir_name )  ) ) == NULL )
      {
        ERROR( "Malloc Failed*ABORT" );
        exit( 1 );
      }

      den_ptr->file      = NULL;
      den_ptr->next      = NULL;
      den_ptr->prev      = NULL;
      den_ptr->sub_tree  = NULL;
      den_ptr->total_bytes    = 0L;
      den_ptr->matching_bytes = 0L;
      den_ptr->tagged_bytes   = 0L;
      den_ptr->total_files    = 0;
      den_ptr->matching_files = 0;
      den_ptr->tagged_files   = 0;
      den_ptr->access_denied  = FALSE;
      den_ptr->up_tree = father_dir_entry;

      statistic.disk_total_directories++;

      (void) strcpy( den_ptr->name, dir_name );
     
      if( STAT( buffer, &stat_struct ) )
      {
	ERROR( "Stat Failed*ABORT" );
	exit( 1 );
      }

      (void) memcpy( &den_ptr->stat_struct, 
		     &stat_struct,
		     sizeof( stat_struct )
		    );
    

      /* Sortieren durch direktes Einfuegen */
      /*------------------------------------*/

      for( des_ptr = father_dir_entry->sub_tree; des_ptr; des_ptr = des_ptr->next )
      {
	if( strcmp( des_ptr->name, den_ptr->name ) > 0 ) 
	{
	  /* des-Element ist groesser */
	  /*--------------------------*/

	  den_ptr->next = des_ptr;
	  den_ptr->prev = des_ptr->prev;
	  if( des_ptr->prev) des_ptr->prev->next = den_ptr;
	  else father_dir_entry->sub_tree = den_ptr;
	  des_ptr->prev = den_ptr;
	  break;
	}
	
	if( des_ptr->next == NULL )
	{
	  /* Ende der Liste erreicht; ==> einfuegen */
	  /*----------------------------------------*/

          den_ptr->prev = des_ptr;
	  den_ptr->next = des_ptr->next;
          des_ptr->next = den_ptr;
	  break;
	}
      }

      if( father_dir_entry->sub_tree == NULL )
      {
	/* Erstes Element */
	/*----------------*/

	father_dir_entry->sub_tree = den_ptr;
	den_ptr->prev = NULL;
	den_ptr->next = NULL;
      }

      (void) GetAvailBytes( &statistic.disk_space );

      result = 0;
    }
  }
 
  move( LINES - 2, 1 ); clrtoeol();

  return( result );
}





