/* Autor        : W. Bregulla
 * Modul        : readtree.c
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : Funktionen zum Lesen des Dateibaumes
 * History
 */


#include "ytree.h"

#if defined( sun ) || defined( linux ) || defined( __NeXT__ )
#include <sys/vfs.h>
#elif defined( WIN32 )
#include <dos.h>
#else
#include <sys/statfs.h>
#endif



#define MIN( a, b ) ( (a) < (b) ? (a) : (b) )


extern char *sys_errlist[];




extern void exit(int);
extern char *sys_errlist[];





/* Volume-Name und freien Plattenplatz ermitteln */
/*-----------------------------------------------*/

int GetDiskParameter( char *path, 
		      char *volume_name, 
		      unsigned long *avail_bytes
		    )
{

#ifdef WIN32
  struct _diskfree_t diskspace;
#else
  struct statfs statfs_struct;
#endif /* WIN32 */

  char *p;
  char *fname;
  int  result;


#ifdef WIN32
  if( ( result = _getdiskfree( 0, &diskspace ) ) == 0 )
#else
  if( ( result = STATFS( path, &statfs_struct, sizeof( statfs_struct ), 0 ) ) == 0 )
#endif /* WIN32 */
  {
    if( mode == DISK_MODE )
    {

#if defined( linux ) || defined( sun ) || defined( HP_UX ) || defined( __NeXT__ )
      fname = "UNIX";
#elif defined ( WIN32 )
      fname = "WIN-NT";
#else
      fname = statfs_struct.f_fname;
#endif

      (void) strncpy( volume_name, 
	              fname,
		      MIN( DISK_NAME_LENGTH, sizeof( fname ) )
		    );
      volume_name[ MIN( DISK_NAME_LENGTH, sizeof( fname ))] = '\0';
    }
    else
    {  
      /* TAR/ZOO/ZIP-FILE_MODE */
      /*-----------------------*/
      
      if( ( p = strrchr( statistic.login_path, '/' ) ) == NULL ) 
        p = statistic.login_path;
      else p++;

      (void) strncpy( volume_name, p, sizeof( statistic.disk_name ) );
      volume_name[sizeof( statistic.disk_name )] = '\0';
    }
#ifdef WIN32
    *avail_bytes = (long) diskspace.bytes_per_sector *
                   (long) diskspace.sectors_per_cluster *
                   (long) diskspace.avail_clusters;
#else
    *avail_bytes = statfs_struct.f_bfree * BLKSIZ;
#endif /* WIN32 */

  }

  return( result );

}




int GetAvailBytes(long unsigned int *avail_bytes)
{
#ifdef WIN32
  struct _diskfree_t diskspace;
#else
  struct statfs statfs_struct;
#endif /* WIN32 */

  int result;

  result = -1;

#ifdef WIN32
  if( !_getdiskfree( 0, &diskspace ) )
  {
    *avail_bytes = (long) diskspace.bytes_per_sector *
                   (long) diskspace.sectors_per_cluster *
                   (long) diskspace.avail_clusters;
    result = 0;
  }
#else

  if( !STATFS( statistic.tree->name, &statfs_struct, sizeof( statfs_struct ), 0 ) ) 
  {
    *avail_bytes = statfs_struct.f_bfree * BLKSIZ;
    result = 0;
  }
#endif

  return( result );
}




/* Dateibaum lesen: path = "Root"-Pfad
 * dir_entry wird von der Funktion gefuellt
 */


int ReadTree(DirEntry *dir_entry, char *path)
{ 
  DIR           *dir;
  struct stat   stat_struct;
  struct dirent *dirent;
  char          new_path[PATH_LENGTH + 1];
  DirEntry      first_dir_entry;
  DirEntry      *des_ptr;
  DirEntry      *den_ptr;
  FileEntry     first_file_entry;
  FileEntry     *fes_ptr;
  FileEntry     *fen_ptr;

  /* dir_entry initialisieren */
  /*--------------------------*/

  dir_entry->file      = NULL;
  dir_entry->next      = NULL;
  dir_entry->prev      = NULL;
  dir_entry->sub_tree  = NULL;
  dir_entry->total_bytes    = 0L;
  dir_entry->matching_bytes = 0L;
  dir_entry->tagged_bytes   = 0L;
  dir_entry->total_files    = 0;
  dir_entry->matching_files = 0;
  dir_entry->tagged_files   = 0;
  dir_entry->access_denied  = FALSE;
  dir_entry->start_file     = 0;
  dir_entry->cursor_pos     = 0;
  dir_entry->global_flag    = FALSE;
  dir_entry->login_flag     = FALSE;

  statistic.disk_total_directories++;

  if( S_ISBLK( dir_entry->stat_struct.st_mode ) )
    return( 0 ); /* Block-Device */

  if( ( dir = opendir( path ) ) == NULL )
  {
    dir_entry->access_denied = TRUE;
    return( 1 );
  }
  
  first_dir_entry.prev  = NULL;
  first_dir_entry.next  = NULL;
  *first_dir_entry.name = '\0';
  first_file_entry.next = NULL;
  fes_ptr               = &first_file_entry;
  
  while( ( dirent = readdir( dir ) ) != NULL )
  {
    if( !strcmp( dirent->d_name, "." ) || !strcmp( dirent->d_name, ".." ) )
      continue;
    
    (void) strcpy( new_path, path );
    if( strcmp( new_path, FILE_SEPARATOR_STRING ) ) 
      (void) strcat( new_path, FILE_SEPARATOR_STRING );
    (void) strcat( new_path, dirent->d_name );
   
    if( STAT( new_path, &stat_struct ) )
    {
      if( errno == EACCES ) continue;
      (void) sprintf( message, "Stat failed on*%s*IGNORED", new_path );
      ERROR( message );
      continue;
    }
     
    if( S_ISDIR( stat_struct.st_mode ) )
    {
      /* Directory-Entry */
      /*-----------------*/

      if( ( den_ptr = (DirEntry *) 
		       malloc( sizeof( DirEntry ) + strlen( dirent->d_name ) ) 
   	   ) == NULL )
      {
        ERROR( "Malloc Failed*ABORT" );
        exit( 1 );
      }

      den_ptr->up_tree = dir_entry;

      (void) strcpy( den_ptr->name, dirent->d_name );


      (void) memcpy( &den_ptr->stat_struct, 
		     &stat_struct,
		     sizeof( stat_struct )
		    );
    
      (void) ReadTree( den_ptr, new_path ); 

      /* Sortieren durch direktes Einfuegen */
      /*------------------------------------*/

      for( des_ptr = &first_dir_entry; des_ptr; des_ptr = des_ptr->next )
      {
	if( strcmp( des_ptr->name, den_ptr->name ) > 0 ) 
	{
	  /* des-Element ist groesser */
	  /*--------------------------*/

	  den_ptr->next = des_ptr;
	  den_ptr->prev = des_ptr->prev;
	  des_ptr->prev->next = den_ptr;
	  des_ptr->prev = den_ptr;
	  break;
	}
	
	if( des_ptr->next == NULL )
	{
	  /* Ende der Liste erreicht; ==> einfuegen */
	  /*----------------------------------------*/

          den_ptr->prev = des_ptr;
	  den_ptr->next = des_ptr->next;
          des_ptr->next = den_ptr;
	  break;
	}

      }
    }
    else
    {
      /* File-Entry */
      /*------------*/
     
      int n;
      char link_path[PATH_LENGTH + 1];

      /* Test, ob Eintrag Symbolischer Link ist */
      /*----------------------------------------*/

      n = 0; *link_path = '\0';

      if( S_ISLNK( stat_struct.st_mode ) )
      {
	/* Ja, symbolischer Name wird an "echten" Namen angehaengt */
	/*---------------------------------------------------------*/

	if( ( n = readlink( new_path, link_path, sizeof( link_path ) ) ) == -1 )
	{
	  (void) sprintf( message, "readlink failed*%s*%s*IGNORED", 
			  new_path,
			  sys_errlist[errno]
			);
	  WARNING( message );
          (void) strcpy( link_path, "unknown" );
	  n = strlen( link_path );
	}
	link_path[n] = '\0';

        if( ( fen_ptr = (FileEntry *) 
		        malloc( sizeof( FileEntry ) + 
			strlen( dirent->d_name )    +
			n + 1 ) 
	    ) == NULL )
        {
          ERROR( "Malloc Failed*ABORT" );
          exit( 1 );
        }
  	
        (void) strcpy( fen_ptr->name, dirent->d_name );
        (void) strcpy( &fen_ptr->name[strlen(fen_ptr->name) + 1], 
		       link_path );
      }
      else
      {
        if( ( fen_ptr = (FileEntry *) 
		        malloc( sizeof( FileEntry ) + strlen( dirent->d_name ) ) 
	    ) == NULL )
        {
          ERROR( "Malloc Failed*ABORT" );
          exit( 1 );
        }
  	
        (void) strcpy( fen_ptr->name, dirent->d_name );
      }

      fen_ptr->next = NULL;
      fen_ptr->prev = NULL;
      fen_ptr->tagged = FALSE;
      
      (void) memcpy( &fen_ptr->stat_struct, 
		     &stat_struct,
		     sizeof( stat_struct )
		    );

      fen_ptr->dir_entry = dir_entry;
      fes_ptr->next      = fen_ptr;
      fen_ptr->prev      = fes_ptr;
      fes_ptr            = fen_ptr;
      dir_entry->total_files++;
      dir_entry->total_bytes += stat_struct.st_size;
      statistic.disk_total_files++;
      statistic.disk_total_bytes += stat_struct.st_size;
    
    }
  }
  
  (void) closedir( dir );

  if( first_file_entry.next ) first_file_entry.next->prev = NULL;
  if( first_dir_entry.next )  first_dir_entry.next->prev = NULL;

  dir_entry->file = first_file_entry.next;
  dir_entry->sub_tree = first_dir_entry.next;

  DisplayDiskStatistic();

  doupdate();

  return( 0 );
}


