/* Projekt      : YTREE
 * Autor        : W. Bregulla
 * Modul        : stat.c
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : Statistik-Modul
 * History
 */


#include "ytree.h"


static void PrettyPrintNumber(int y, int x, long unsigned int number);


void DisplayDiskStatistic(void)
{
  mvprintw( 2,  COLS - 15, "%-14s", statistic.file_spec );
  mvprintw( 4,  COLS - 15, "%-14s", statistic.disk_name );
  PrettyPrintNumber( 6,  COLS - 13, statistic.disk_space );
  mvprintw( 8,  COLS - 21, "%-20s", "DISK Statistics" );
  PrettyPrintNumber( 10, COLS - 13, statistic.disk_total_files );
  PrettyPrintNumber( 11, COLS - 13, statistic.disk_total_bytes );
  PrettyPrintNumber( 13, COLS - 13, statistic.disk_matching_files );
  PrettyPrintNumber( 14, COLS - 13, statistic.disk_matching_bytes );
  PrettyPrintNumber( 16, COLS - 13, statistic.disk_tagged_files );
  PrettyPrintNumber( 17, COLS - 13, statistic.disk_tagged_bytes );
  mvprintw( 18, COLS - 20, "%-19s", "Current Directory" );

  wnoutrefresh( stdscr );
}



void DisplayAvailBytes(void)
{
  PrettyPrintNumber( 6,  COLS - 13, statistic.disk_space );
  wnoutrefresh( stdscr );
}




void DisplayFileSpec(void)
{
  mvprintw( 2,  COLS - 15, "%-14s", statistic.file_spec );

  wnoutrefresh( stdscr );
}




void DisplayDirStatistic(DirEntry *dir_entry)
{
  char format[10];
  char buffer[PATH_LENGTH + 1];

  (void) sprintf( format, "%%-%ds", COLS - 7 );

  (void) GetPath( dir_entry, statistic.path );
  mvprintw( 0,  7,  format, FormFilename( buffer, statistic.path, COLS - 7 ) );
  mvprintw( 8,  COLS - 21, "%-20s", "DIR Statistics" );
  PrettyPrintNumber( 10, COLS - 13, dir_entry->total_files );
  PrettyPrintNumber( 11, COLS - 13, dir_entry->total_bytes );
  PrettyPrintNumber( 13, COLS - 13, dir_entry->matching_files );
  PrettyPrintNumber( 14, COLS - 13, dir_entry->matching_bytes );
  PrettyPrintNumber( 16, COLS - 13, dir_entry->tagged_files );
  PrettyPrintNumber( 17, COLS - 13, dir_entry->tagged_bytes );
  mvprintw( 18, COLS - 20, "%-19s", "Current File" );

  wnoutrefresh( stdscr );
}





void DisplayDirTagged(DirEntry *dir_entry)
{
  PrettyPrintNumber( 16, COLS - 13, dir_entry->tagged_files );
  PrettyPrintNumber( 17, COLS - 13, dir_entry->tagged_bytes );
  
  wnoutrefresh( stdscr );
}



void DisplayDiskTagged(void)
{
  PrettyPrintNumber( 16, COLS - 13, statistic.disk_tagged_files );
  PrettyPrintNumber( 17, COLS - 13, statistic.disk_tagged_bytes );
  
  wnoutrefresh( stdscr );
}



void DisplayDirParameter(DirEntry *dir_entry)
{
  char *p, *f;
  char format[10];
  char buffer[PATH_LENGTH + 1];

  p = strrchr( dir_entry->name, FILE_SEPARATOR_CHAR ); 
 
  if( p == NULL ) f = dir_entry->name;
  else            f = p + 1;
 
  (void) sprintf( format, "%%-%ds", COLS - 7 );
  (void) GetPath( dir_entry, statistic.path );
  mvprintw( 0,  7,  format, FormFilename( buffer, statistic.path, COLS - 7 ) );
  mvprintw( 19, COLS - 19, "%-14s", CutFilename( buffer, f, 14 ) );
  PrettyPrintNumber( 20, COLS - 13, (unsigned long) dir_entry->total_bytes );
  
  wnoutrefresh( stdscr );
}






void DisplayGlobalFileParameter(FileEntry *file_entry)
{
  char buffer1[PATH_LENGTH+1];
  char buffer2[PATH_LENGTH+1];
  char format[10];

  (void) sprintf( format, "%%-%ds", COLS - 7 );
  (void) GetPath( file_entry->dir_entry, buffer1 );
  mvprintw( 0,  7,  format, FormFilename( buffer2, buffer1, COLS - 7 ) );
  mvprintw( 19, COLS - 19, "%-14s", CutFilename( buffer1, file_entry->name, 14 ) );
  PrettyPrintNumber( 20, COLS - 13, (unsigned long) file_entry->stat_struct.st_size );

  wnoutrefresh( stdscr );
}




void DisplayFileParameter(FileEntry *file_entry)
{
  char buffer[14+1];
  mvprintw( 19, COLS - 19, "%-14s", CutFilename( buffer, file_entry->name, 14 ) );
  PrettyPrintNumber( 20, COLS - 13, (unsigned long) file_entry->stat_struct.st_size );

  wnoutrefresh( stdscr );
}



static void PrettyPrintNumber(int y, int x, long unsigned int number)
{
  int million, thousand, one;

  million  = (int) ( number / 1000000L );
  thousand = (int) ( ( number % 1000000L ) / 1000L );
  one      = (int) ( number % 1000L );

  if( million ) mvprintw( y, x, "%3ld.%03ld.%03ld", million, thousand, one);
  else if( thousand ) mvprintw( y, x, "    %3ld.%03ld", thousand, one);
  else mvprintw( y, x, "        %3ld", one );
}



