/* Autor        : W. Bregulla
 * Modul        : util.c
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : Diverse Hilfsfunktionen
 * History
 */


#include "ytree.h"


extern char *sys_errlist[];

typedef struct
{
  char *extension;
  int  compress_method;
} Extension2Method;


static Extension2Method compress_extensions[] = COMPRESS_EXTENSIONS;



char *GetPath(DirEntry *dir_entry, char *buffer)
{
  DirEntry *de_ptr;
  char     help_buffer[PATH_LENGTH + 1];

  *buffer = '\0';

  for( de_ptr = dir_entry; de_ptr; de_ptr = de_ptr->up_tree )
  {
    *help_buffer = '\0';
    if( de_ptr->up_tree ) (void) strcat( help_buffer, FILE_SEPARATOR_STRING );
    if( strcmp( de_ptr->name, FILE_SEPARATOR_STRING ) )
      (void) strcat( help_buffer, de_ptr->name );
    (void) strcat( help_buffer, buffer );
    (void) strcpy( buffer, help_buffer );
  }

  /* if( *buffer == '\0' ) (void) strcpy( buffer, FILE_SEPARATOR_STRING ); */

  return( buffer );
}





char *GetFileNamePath(FileEntry *file_entry, char *buffer)
{
  (void) GetPath( file_entry->dir_entry, buffer );
  if( *buffer && strcmp( buffer, FILE_SEPARATOR_STRING ) ) 
    (void) strcat( buffer, FILE_SEPARATOR_STRING );
  return( strcat( buffer, file_entry->name ) );
}



char *GetRealFileNamePath(FileEntry *file_entry, char *buffer)
{
  char *sym_name;

  if( mode == DISK_MODE )
    return( GetFileNamePath( file_entry, buffer ) );

  if( S_ISLNK( file_entry->stat_struct.st_mode ) )
  {
    sym_name = &file_entry->name[ strlen( file_entry->name ) + 1 ];
    if( *sym_name == FILE_SEPARATOR_CHAR )
      return( strcpy( buffer, sym_name ) );
  }
  
  (void) GetPath( file_entry->dir_entry, buffer );
  if( *buffer && strcmp( buffer, FILE_SEPARATOR_STRING ) ) 
    (void) strcat( buffer, FILE_SEPARATOR_STRING );
  if( S_ISLNK( file_entry->stat_struct.st_mode ) )
    return( strcat( buffer, &file_entry->name[ strlen( file_entry->name ) + 1 ] ) );
  else
    return( strcat( buffer, file_entry->name ) );
}






int GetDirEntry(DirEntry *tree, DirEntry *current_dir_entry, char *dir_path, DirEntry **dir_entry, char *to_path)
{
  char dest_path[PATH_LENGTH+1];
  char current_path[PATH_LENGTH+1];
  char help_path[PATH_LENGTH+1];
  char *token;
  DirEntry *de_ptr, *sde_ptr;
  int n;

  *dir_entry = NULL;
  *to_path   = '\0';

  if( getcwd( current_path, sizeof( current_path ) - 2 ) == NULL )
  {
    (void) sprintf( message, "getcwd failed*%s", sys_errlist[errno] );
    ERROR( message );
    return( -1 );
  }

  if( *dir_path != FILE_SEPARATOR_CHAR ) 
  {
    if( chdir( GetPath( current_dir_entry, help_path ) ) )
    {
      ERROR( "Chdir Failed" );
      return( -1 );
    }
  }

  if( chdir( dir_path ) )
  {
    (void) sprintf( message, "Invalid Path!*\"%s\"", dir_path );
    MESSAGE( message );
    return( -1 );
  }

  (void) getcwd( dest_path, sizeof( dest_path ) - 2 );
  (void) strcpy( to_path, dest_path );

  if( chdir( current_path ) )
  {
    ERROR( "Chdir failed; Can't resume" );
    return( -1 );
  }

  n = strlen( tree->name );
  if( !strncmp( tree->name, dest_path, n ) &&
      ( dest_path[n] == FILE_SEPARATOR_CHAR || dest_path[n] == '\0' ) )
  {
    /* Pfad befindet sich im (Sub)-Tree */
    /*----------------------------------*/

    de_ptr = tree;
    token = strtok( &dest_path[n], FILE_SEPARATOR_STRING );
    while( token )
    {
      for( sde_ptr = de_ptr->sub_tree; sde_ptr; sde_ptr = sde_ptr->next )
      {
        if( !strcmp( sde_ptr->name, token ) )
	{
	  /* Subtree gefunden */
	  /*------------------*/

	  de_ptr = sde_ptr;
	  break;
	}
      }
      if( sde_ptr == NULL )
      {
	(void) sprintf( message, "Can't find directory; token=%s", token );
	ERROR( message );
	return( -1 );
      }
      token = strtok( NULL, FILE_SEPARATOR_STRING );
    }
    *dir_entry = de_ptr;
  }
  return( 0 );
}




int GetFileEntry(DirEntry *de_ptr, char *file_name, FileEntry **file_entry)
{
  FileEntry *fe_ptr;

  *file_entry = NULL;

  for( fe_ptr = de_ptr->file; fe_ptr; fe_ptr = fe_ptr->next )
  {
    if( !strcmp( fe_ptr->name, file_name ) )
    {
      /* Eintrag gefunden */
      /*------------------*/

      *file_entry = fe_ptr;
      break;
    }
  }
  return( 0 );
}





char *GetAttributes(ushort modus, char *buffer)
{
  char *save_buffer = buffer;

       if( S_ISREG( modus ) )  *buffer++ = '-';
  else if( S_ISDIR( modus ) )  *buffer++ = 'd';
  else if( S_ISCHR( modus ) )  *buffer++ = 'c';
  else if( S_ISBLK( modus ) )  *buffer++ = 'b';
  else if( S_ISFIFO( modus ) ) *buffer++ = 'p';
  else if( S_ISLNK( modus ) )  *buffer++ = 'l';
  else if( S_ISSOCK( modus ) ) *buffer++ = 's';  /* ??? */
  else                         *buffer++ = '?';  /* unknown */

  if( modus & S_IRUSR ) *buffer++ = 'r';
  else *buffer++ = '-';

  if( modus & S_IWUSR ) *buffer++ = 'w';
  else *buffer++ = '-';
    
  if( modus & S_IXUSR ) *buffer++ = 'x';
  else *buffer++ = '-';
    
  if( modus & S_ISUID ) *(buffer - 1) = 's';
    

  if( modus & S_IRGRP ) *buffer++ = 'r';
  else *buffer++ = '-';

  if( modus & S_IWGRP ) *buffer++ = 'w';
  else *buffer++ = '-';
    
  if( modus & S_IXGRP ) *buffer++ = 'x';
  else *buffer++ = '-';
    
  if( modus & S_ISGID ) *(buffer - 1) = 's';


  if( modus & S_IROTH ) *buffer++ = 'r';
  else *buffer++ = '-';

  if( modus & S_IWOTH ) *buffer++ = 'w';
  else *buffer++ = '-';
    
  if( modus & S_IXOTH ) *buffer++ = 'x';
  else *buffer++ = '-';
    
  *buffer = '\0';

  return( save_buffer );
}





/***************************************************************************
 * InputStr                                                                *
 * Liest eine Zeichenkette an Position (y,x) mit der max. Laenge length    *
 * Vorschlagswert fuer die Eingabe ist s selbst                            *
 * Zurueckgegeben wird das Zeichen, mit dem die Eingabe beendet wurde      *
 ***************************************************************************/


int InputString(char *s, int y, int x, int length, char *term)
                               /* Ein- und Ausgabestring              */
                               /* Position auf Bildschirm             */
                               /* max. Laenge                         */
                               /* Menge von Terminierungszeichen      */
{
  int p;                       /* Aktuelle Position                   */
  int c1;                      /* Gelesenes Zeichen                   */
  int i;                       /* Laufvariable                        */
  BOOL key_pressed = FALSE;

  static BOOL insert_flag = TRUE;

  /* Feld gefuellt ausgeben */
  /*------------------------*/

  mvaddstr( y, x, s );
  
  for(i=strlen(s); i < length; i++)
    addch( '_' );


  p = 0;

  do  /* until termkey pressed */
  {
    move( y, x+p );

    wnoutrefresh( stdscr );
    doupdate();

    c1 = getch();

    if( c1 >= ' ' && c1 < 0x7F )
    {
      /* Darstellbares Zeichen */
      /*-----------------------*/

      if( key_pressed == FALSE )
      {
	s[0] = '\0';
        
	move( y, x );
        for(i=0; i < length; i++)
          addch( '_' );
	move( y, x );
      }

      if( p < length )
      {
        /* Platz zum einfuegen vorhanden */
        /*-------------------------------*/

        if( insert_flag )
        {
          /* Restliche Zeichen eine Position vorruecken */
          /*--------------------------------------------*/

          if( strlen( s ) == length ) s[length-1] = '\0';
          for(i=strlen(s); i >= p; i-- )
            s[i+1] = s[i];
        }
        else
        {
          /* Simples ueberschreiben */
          /*------------------------*/

          if( s[p] == '\0' )
            s[p+1] = '\0';
        }

        s[p] = (char) c1;

        addstr( &s[p] );
        p++;
      }
      else
      {
        /* Zeile voll */
        /*------------*/

        beep();
      }
    }
    else
    {
      /* Steuerzeichen */
      /*---------------*/

      switch( c1 )
      {
        case KEY_LEFT       : if( p > 0 )
                                p--;
                              else
                                beep();
                              break;
        case KEY_RIGHT      : if( p < (int) strlen( s ) )
                                p++;
                              else
                                beep();
                              break;
        case KEY_HOME       : p = 0;
                              break;
        case KEY_END        : p = strlen( s );
                              break;
        case KEY_DC         : if( p < (int) strlen( s ) )
                              {
                                (void) strcpy( s+p, s+p+1 );
                                printw( "%s_", s+p );
                              }
                              break;
	case 0x08           :
        case 0x7F           :
        case KEY_BACKSPACE  : if( p > 0 )
                              {
                                (void) strcpy( s+p-1, s+p );
                                printw( "\b%s_", s+p-1 );
                                p--;
                              }
                              else
                                beep();
                              break;
        case KEY_DL         : for(i=0; i < (int) strlen(s) - p; i++)
			        addch( '_' );
                              s[p] = '\0';
                              break;
	case KEY_EIC        :
        case KEY_IC         : insert_flag ^= TRUE; 
                              break;
        default:              if( !strrchr( term, c1 ) )
                                beep();
                              break;
      }
    }
    key_pressed = TRUE;
  } while( !strrchr( term, c1 ) );

  p = strlen( s );

  move( y, x + p );

  for(i=0; i < length - p; i++ )
   addch( ' ' );

  return( c1 );
}





int InputChoise(char *msg, char *term)
{
  int  c;

  ClearHelp();

  mvprintw( LINES - 2, 1, msg );
  wnoutrefresh( stdscr );
  doupdate();
  do
  {
    c = getch();
    if( islower( c ) ) c = toupper( c );
  } while( !strchr( term, c ) );
  
  echochar( c );

  move( LINES - 2, 1 ); clrtoeol();

  return( c );
}






char *CTime(time_t f_time, char *buffer)
{
  char   *cptr;
  time_t now;

  if( (now = time( NULL )) == -1 )
  {
    ERROR( "time() failed" );
    exit( 1 );
  }

  cptr = ctime( &f_time );
  (void) strncpy( buffer, cptr+4, 12 );
  buffer[12] = '\0';

  if( (now - f_time) > 31536000L )
  {
    /* Differenz groesser als 1 Jahr */
    /*-------------------------------*/

    (void) strncpy( &buffer[7], cptr + 19, 5 );
    
  }

  return( buffer );
}

  


void PrintSpecialString(WINDOW *win, int y, int x, char *str)
{
  int ch;

  wmove( win, y, x );
  for( ; *str; str++ )
  {
    switch( *str )
    {
      case '1': ch = ACS_ULCORNER; break;
      case '2': ch = ACS_URCORNER; break;
      case '3': ch = ACS_LLCORNER; break;
      case '4': ch = ACS_LRCORNER; break;
      case '5': ch = ACS_TTEE;     break;
      case '6': ch = ACS_LTEE;     break;
      case '7': ch = ACS_RTEE;     break;
      case '8': ch = ACS_BTEE;     break;
      case '|': ch = ACS_VLINE;    break;
      case '-': ch = ACS_HLINE;    break;
      default:  ch = (int) *str;
    }
    waddch( win, ch );
  }
}



/*****************************************************************************
 *                              FormFilename                                 *
 *****************************************************************************/

char *FormFilename(char *dest, char *src, unsigned int max_len)
{
  int i;
  int begin;
  unsigned int l;

  l = strlen(src);

  if( l <= max_len )
    return( strcpy( dest, src ) );
  else
  {
    for(i=0; i < (int) max_len - 4; i++)
      if( src[l - i] == FILE_SEPARATOR_CHAR || src[l - i] == '\\' )
        begin = l - i;
    (void) strcpy( dest, "/..." );
    return( strcat(dest, &src[begin] ) );
  }
}


/*****************************************************************************
 *                              CutFilename                                  *
 *****************************************************************************/

char *CutFilename(char *dest, char *src, unsigned int max_len)
{
  unsigned int l;

  l = strlen(src);

  if( l <= max_len )
    return( strcpy( dest, src ) );
  else
  {
    (void) strncpy( dest, src, max_len - 3 );
    (void) strcpy( &dest[max_len - 3], "..." );
    return( dest );
  }
}



/*****************************************************************************
 *                                  Fnsplit                                  *
 *****************************************************************************/

/* Aufsplitten des Dateinamens in die einzelnen Komponenten */

void Fnsplit(char *path, char *dir, char *name)
{
  int  i;
  char *name_begin;
  char *trunc_name;

  while( *path == ' ' || *path == '\t' ) path++;

  while( strchr(path, FILE_SEPARATOR_CHAR ) || strchr(path, '\\') )
    *(dir++) = *(path++);

  *dir = '\0';
  
  name_begin = path;
  trunc_name = name;

  for(i=0; i < PATH_LENGTH && *path; i++ ) 
    *(name++) = *(path++);

  *name = '\0';

  if( i == PATH_LENGTH && *path )
  {
    (void) sprintf( message, "filename too long:*%s*truncating to*%s", 
		    name_begin, trunc_name
		  );
    WARNING( message );
  }
}



void HitReturnToContinue(void)
{
  vidattr( A_REVERSE );
  putp( "[Hit return to continue]" );
  vidattr( 0 );
  (void) fflush( stdout );
  (void) getch();
  doupdate();
}




int BuildFilename( char *in_filename,
		   char *pattern,
		   char *out_filename
		 )
{
  char *cptr;
  int  result = 0;


  for( ; *pattern; pattern++ )
  {
    if( *pattern == '*' )
    {
      cptr = in_filename;
      for( ; *out_filename = *cptr; out_filename++, cptr++ );
    }
    else
    {
      *out_filename++ = *pattern;
    }
  }

  *out_filename = '\0';

  return( result );
}




int GetCompressMethod( char *filename )
{
  int i;
  char *extension;

  extension = strrchr( filename, '.' );

  if( extension == NULL ) return( NO_COMPRESS );

  for( i=0; 
       i < sizeof( compress_extensions ) / sizeof( compress_extensions[0] ); 
       i++ 
     )
  {
    if( !strcmp( extension, compress_extensions[i].extension ) )
      return( compress_extensions[i].compress_method );
  }

  return( NO_COMPRESS );
}



#ifdef __NeXT__

char *getcwd(char *dest, int len)
{
  static char  buffer[MAXNAMLEN];
  DIR    *dirp;
  struct direct  *dp;
  long   inode;
  char   *cp = buffer + MAXNAMLEN - 1;

  *cp = 0;

  do {
    dirp = opendir(".");
    for (dp=readdir(dirp); dp; dp=readdir(dirp)) {
      if(dp->d_namlen && !strcmp(dp->d_name,".")) {
        break;
      }
    }
    if (inode == dp->d_ino) break;
    inode = dp->d_ino;
    (void) closedir(dirp);
    dirp = opendir("..");
    for (dp=readdir(dirp); dp && (dp->d_ino != inode); dp=readdir(dirp));
    cp -= dp->d_namlen;
    (void) strncpy(cp,dp->d_name,dp->d_namlen);
    *--cp = FILE_SEPARATOR_CHAR;
    (void) closedir(dirp);
  } while(!chdir(".."));
  
  (void) chdir(cp+2);
  (void) strncpy(dest,cp+2,len);
}


#endif


#ifdef VI_KEYS

int ViKey( int ch )
{
  switch( ch )
  {
    case VI_KEY_UP:    ch = KEY_UP;    break;
    case VI_KEY_DOWN:  ch = KEY_DOWN;  break;
    case VI_KEY_RIGHT: ch = KEY_RIGHT; break;
    case VI_KEY_LEFT:  ch = KEY_LEFT;  break;
    case VI_KEY_PPAGE: ch = KEY_PPAGE; break;
    case VI_KEY_NPAGE: ch = KEY_NPAGE; break;
  }
  return(ch);
}

#endif /* VI_KEYS */

  
