/* Projekt      : YTREE
 * Autor        : W. Bregulla
 * Modul        : view.c
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : View-Kommando-Bearbeitung
 * History
 */


#include "ytree.h"


#define ISMANEXT( c )  ( isdigit( c ) || ( (c) == 'n' ) || ( (c) == 'm' ) )


extern void exit(int);
extern char *sys_errlist[];

static int ViewFile(char *file_path);
static int ViewArchiveFile(char *file_path);




int View(char *file_path)
{
  switch( mode )
  {
    case DISK_MODE :     return( ViewFile( file_path ) );
    case TAR_FILE_MODE : 
    case ZOO_FILE_MODE :
    case ZIP_FILE_MODE :
    case LHA_FILE_MODE :
    case ARC_FILE_MODE : return( ViewArchiveFile( file_path ) );
    default:             beep(); return( -1 );
  }
}




static int ViewFile(char *file_path)
{
  char *command_line;
  int  compress_method;
  int  result = -1;
  int  l;

  if( access( file_path, R_OK ) )
  {
    (void) sprintf( message, 
		    "View not possible!*\"%s\"*%s", 
		    file_path, 
		    sys_errlist[errno] 
		  );
    MESSAGE( message );
    ESCAPE;
  }

  if( ( command_line = malloc( COMMAND_LINE_LENGTH + 1 ) ) == NULL )
  {
    ERROR( "Malloc failed*ABORT" );
    exit( 1 );
  }

  compress_method = GetCompressMethod( file_path );

  l = strlen( file_path );

  if( compress_method == FREEZE_COMPRESS )
  {
    (void) sprintf( command_line, 
		    "%s < %s | %s %s", 
		    MELT, 
		    file_path, 
		    PAGER, 
		    PAGEOPT
		  );
  }
  else if( compress_method == COMPRESS_COMPRESS )
  {
    (void) sprintf( command_line, 
		    "%s < %s | %s %s", 
		    UNCOMPRESS, 
		    file_path, 
		    PAGER, 
		    PAGEOPT
		  );
  }
  else if( compress_method == GZIP_COMPRESS )
  {
    (void) sprintf( command_line, 
                    "%s < %s | %s %s", 
		    GNUUNZIP, 
		    file_path, 
		    PAGER, 
		    PAGEOPT
		  );
  }
  else if( l > 3 && 
	   file_path[l - 2] == '.' &&
	   ISMANEXT( file_path[l - 1] ) &&
	   file_path[ l ] == '\0' )
  {
    (void) sprintf( command_line, 
		    "%s < %s | %s %s", 
		    MANROFF, 
		    file_path, 
		    PAGER, 
		    PAGEOPT
		  );
  }
  else
  {
    (void) sprintf( command_line, 
		    "%s %s %s", 
		    PAGER, 
		    PAGEOPT, 
		    file_path 
		  );
  }

  result = SystemCall( command_line );

  free( command_line );

FNC_XIT:

  return( result );
}



static int ViewArchiveFile(char *file_path)
{
  char *command_line;
  char buffer[100];
  int  l;
  int  result = -1;

  if( ( command_line = malloc( COMMAND_LINE_LENGTH + 1 ) ) == NULL )
  {
    ERROR( "Malloc failed*ABORT" );
    exit( 1 );
  }

  l = strlen( file_path );
  
  if( l > 3 && file_path[l - 2] == '.' && ISMANEXT( file_path[l - 1] ) )
  {
    (void) sprintf( buffer, "| %s | %s %s", MANROFF, PAGER, PAGEOPT);
  }
  else
  {
    (void) sprintf( buffer, "| %s %s", PAGER, PAGEOPT );
  }


  MakeExtractCommandLine( command_line, 
			  statistic.login_path, 
			  file_path, 
			  buffer
			);

  result = SystemCall( command_line );

  free( command_line );

  return( result );
}


