/* Projekt      : YTREE
 * Autor        : W. Bregulla
 * Modul        : ytree.h
 * Datum        : 10.07.91
 * Version      : 1.0
 * Beschreibung : Header-Datei fuer YTREE
 * History
 */
#define linux

#ifdef __NeXT
#define TERMCAP
#endif

#include <stdio.h>
#include <ctype.h>

#ifdef _IBMR2
#define NLS
#endif /* _IBMR2 */

#ifdef NCURSES
#include <ncurses.h>
#else
#include <curses.h>
#endif

#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>


#if __STDC__ || defined( _IBMR2 )
#include <stdlib.h>
#endif /* __STDC__ || _IBMR2 */

#ifdef __NeXT__
extern char *getcwd();
#include <sys/dir.h>
#define dirent direct
#else
#include <unistd.h>
#include <dirent.h>
#endif /* __NeXT__ */

#ifdef  SCO
#include <sys/unistd.h>
#include <sys/ndir.h>
#endif

#include <errno.h>
#include <fcntl.h>
#include <grp.h>
#include <malloc.h>
#include <memory.h>
#include <pwd.h>
#include <setjmp.h>
#include <signal.h>
#include <string.h>

#ifdef TERMCAP
#include <termcap.h> 
#endif


#define STATFS(a, b, c, d )     statfs( a, b, c, d )

#define  sys_errlist     _sys_errlist

#ifdef WIN32

#define  S_IREAD         S_IRUSR
#define  S_IWRITE        S_IWUSR
#define  S_IEXEC         S_IXUSR

#define  popen           _popen
#define  pclose          _pclose
#define  sys_errlist     _sys_errlist

#endif /* WIN32 */

#ifdef __NeXT__

#define  S_IRUSR        S_IREAD
#define  S_IWUSR        S_IWRITE
#define  S_IXUSR        S_IEXEC
#define  S_IRGRP        (S_IREAD >> 3)
#define  S_IWGRP        (S_IWRITE >> 3)
#define  S_IXGRP        (S_IEXEC >> 3)
#define  S_IROTH        (S_IREAD >> 6)
#define  S_IWOTH        (S_IWRITE >> 6)
#define  S_IXOTH        (S_IEXEC >> 6)
#define  S_IRWXO        (S_IROTH|S_IWOTH|S_IXOTH)
#define  S_IRWXG        (S_IRGRP|S_IWGRP|S_IXGRP)
#define  S_IRWXU        (S_IRUSR|S_IWUSR|S_IXUSR)

#endif /* __NeXT__ */


#ifdef linux

#ifdef NCURSES

/* gibt's unter ncurses 0.7 nicht ... */

#define  echochar( ch )              { addch( ch ); refresh(); }
#define  typeahead( file )

#else

#define  TERMCAP

#endif /* NCURSES */

#undef  STATFS
#define STATFS( a, b, c, d )         statfs( a, b )

#endif /* linux */



#ifdef WIN32

/* Diese Funktionen koennen direkt umgesetzt werden */
/*--------------------------------------------------*/

#define  echochar( ch )              { addch( ch ); refresh(); }
#define  putp( str )                 puts( str )


/* ... hier ist ein wenig mehr Arbeit noetig ... */
/*-----------------------------------------------*/

#define  vidattr( attr )


/* ... und hier gibt's keine entsprechende Funktion. */
/*---------------------------------------------------*/

#define  typeahead( file )

#endif /* WIN32 */

#ifdef TERMCAP

#define  KEY_BTAB         5000
#define  KEY_DOWN         5001
#define  KEY_UP           5002
#define  KEY_LEFT         5003
#define  KEY_RIGHT        5004
#define  KEY_END          5005
#define  KEY_HOME         5006
#define  KEY_NPAGE        5007
#define  KEY_PPAGE        5008
#define  KEY_DC           5009
#define  KEY_BACKSPACE    5010
#define  KEY_EIC          5011
#define  KEY_IC           5012
#define  KEY_DL           5013

#define  NO_HIGHLIGHT

#ifndef SCO
#define  A_REVERSE        1 
#define  A_BLINK          2
#endif

#define  BELL             0x07


/* Diese Funktionen koennen direkt umgesetzt werden */
/*--------------------------------------------------*/

#undef   cbreak  
#define  cbreak()                    raw()
#define  beep()                      putchar( BELL )
#define  echochar( ch )              { addch( ch ); refresh(); }
#define  putp( str )                 tputs( str, 1, putchar )
#define  wnoutrefresh( win )         wrefresh( win )


/* ... hier ist ein wenig mehr Arbeit noetig ... */
/*-----------------------------------------------*/

#define  wgetch( win )               TermcapWgetch( win )
#define  vidattr( attr )             TermcapVidattr( attr )
#define  initscr()                   TermcapInitscr()
#define  endwin()                    TermcapEndwin()


/* ... und hier gibt's keine entsprechende Funktion. */
/*---------------------------------------------------*/

#define  doupdate() 
#define  wattrset( win, attr ) 
#define  typeahead( file )
#define  keypad( win, flag )

#endif /* TERMCAP */


#if defined( WIN32 ) || defined( HP_UX ) || defined( _IBMR2 ) || defined( SCO )
#define KEY_BTAB  0x1d
#define KEY_END   KEY_EOL
#endif

#ifdef _IBMR2
#define echochar( c )  { addch( c ); refresh(); }
#endif


#ifdef S_IFLNK
#define STAT(a, b) lstat(a, b)
#else
#define STAT(a, b) stat(a, b)
#define readlink( a, b, c )  (-1)
#endif /* S_IFLNK */

#ifndef S_ISREG
#define S_ISREG( mode )   (((mode) & S_IFMT) == S_IFREG)
#endif /* S_ISREG */

#ifndef S_ISDIR
#define S_ISDIR( mode )   (((mode) & S_IFMT) == S_IFDIR)
#endif /* S_ISDIR */

#ifndef S_ISCHR
#define S_ISCHR( mode )   (((mode) & S_IFMT) == S_IFCHR)
#endif /* S_ISCHR */

#ifndef S_ISBLK
#define S_ISBLK( mode )   (((mode) & S_IFMT) == S_IFBLK)
#endif /* S_ISBLK */

#ifndef S_ISFIFO
#define S_ISFIFO( mode )   (((mode) & S_IFMT) == S_IFIFO)
#endif /* S_ISFIFO */

#ifndef S_ISLNK
#ifdef  S_IFLNK
#define S_ISLNK( mode )   (((mode) & S_IFMT) == S_IFLNK)
#else
#define S_ISLNK( mode )   FALSE
#endif /* S_IFLNK */
#endif /* S_ISLNK */

#ifndef S_ISSOCK
#ifdef  S_IFSOCK
#define S_ISSOCK( mode )   (((mode) & S_IFMT) == S_IFSOCK)
#else
#define S_ISSOCK( mode )   FALSE
#endif /* S_IFSOCK */
#endif /* S_ISSOCK */


#define VI_KEY_UP    'k'
#define VI_KEY_DOWN  'j'
#define VI_KEY_RIGHT 'l'
#define VI_KEY_LEFT  'h'
/* #define VI_KEY_NPAGE CTRL( 'D' )
#define VI_KEY_PPAGE CTRL( 'U' )   0x04 0x15 */
#define VI_KEY_NPAGE 0x04 
#define VI_KEY_PPAGE 0x15 


#define OWNER_NAME_MAX  14
#define GROUP_NAME_MAX  14



/* Sonderzeichen fuer Liniengrafik */
/*---------------------------------*/

#ifndef ACS_ULCORNER
#define ACS_ULCORNER '+'
#endif
#ifndef ACS_URCORNER
#define ACS_URCORNER '+'
#endif
#ifndef ACS_LLCORNER
#define ACS_LLCORNER '+'
#endif
#ifndef ACS_LRCORNER
#define ACS_LRCORNER '+'
#endif
#ifndef ACS_VLINE   
#define ACS_VLINE    '|'
#endif
#ifndef ACS_HLINE   
#define ACS_HLINE    '-'
#endif
#ifndef ACS_RTEE
#define ACS_RTEE    '+'
#endif
#ifndef ACS_LTEE
#define ACS_LTEE    '+'
#endif
#ifndef ACS_BTEE
#define ACS_BTEE    '+'
#endif
#ifndef ACS_TTEE
#define ACS_TTEE    '+'
#endif


/* User-Defines */
/*--------------*/

#include "config.h"


/* Auswahl der benutzten UNIX-Kommandos */
/*--------------------------------------*/

#define CAT     ( getenv( "CAT" )      ? getenv( "CAT" )     : DEFAULT_CAT )
#define HEXDUMP ( getenv( "HEXDUMP" )  ? getenv( "HEXDUMP" ) : DEFAULT_HEXDUMP )
#define EDITOR  ( getenv( "EDITOR" )   ? getenv( "EDITOR" )  : DEFAULT_EDITOR )
#define PAGER   ( getenv( "PAGER" )    ? getenv( "PAGER" )   : DEFAULT_PAGER )
#define PAGEOPT ( getenv( "PAGEOPT" )  ? getenv( "PAGEOPT" ) : DEFAULT_PAGEOPT )
#define MELT    ( getenv( "MELT" )     ? getenv( "MELT" )    : DEFAULT_MELT )
#define UNCOMPRESS  ( getenv( "UNCOMPRESS" )  ? getenv( "UNCOMPRESS" )  : DEFAULT_UNCOMPRESS )
#define GNUUNZIP  ( getenv( "GNUUNZIP" )  ? getenv( "GNUUNZIP" )  : DEFAULT_GNUUNZIP )
#define MANROFF ( getenv( "MANROFF" )  ? getenv( "MANROFF" ) : DEFAULT_MANROFF )

/* Achtung ! Hier muss (?) GNU-Tar benutzt werden, da
 * nur GNU-Tar in der Lage ist auf stdout zu extrahieren (Option O) !!!
 */

#define TARLIST ( getenv( "TARLIST" ) ? getenv( "TARLIST" ) : DEFAULT_TARLIST )
#define TAREXPAND ( getenv( "TAREXPAND" ) ? getenv( "TAREXPAND" ) : DEFAULT_TAREXPAND )

#define ZOOLIST ( getenv( "ZOOLIST" ) ? getenv( "ZOOLIST" ) : DEFAULT_ZOOLIST )
#define ZOOEXPAND ( getenv( "ZOOEXPAND" ) ? getenv( "ZOOEXPAND" ) : DEFAULT_ZOOEXPAND )

#define ZIPLIST ( getenv( "ZIPLIST" ) ? getenv( "ZIPLIST" ) : DEFAULT_ZIPLIST )
#define ZIPEXPAND ( getenv( "ZIPEXPAND" ) ? getenv( "ZIPEXPAND" ) : DEFAULT_ZIPEXPAND )

#define LHALIST ( getenv( "LHALIST" ) ? getenv( "LHALIST" ) : DEFAULT_LHALIST )
#define LHAEXPAND ( getenv( "LHAEXPAND" ) ? getenv( "LHAEXPAND" ) : DEFAULT_LHAEXPAND )

#define ARCLIST ( getenv( "ARCLIST" ) ? getenv( "ARCLIST" ) : DEFAULT_ARCLIST )
#define ARCEXPAND ( getenv( "ARCEXPAND" ) ? getenv( "ARCEXPAND" ) : DEFAULT_ARCEXPAND )

#define DEFAULT_TREE ( getenv( "HOME" ) ? getenv( "HOME" ) : "." )


#define ERROR( msg )   Error( msg, __FILE__, __LINE__ ) 
#define WARNING( msg ) Warning( msg ) 
#define MESSAGE( msg ) Message( msg ) 

#define TAGGED_SYMBOL '*'
#define MAX_MODES      7
#define DISK_MODE      0
#define LL_FILE_MODE   1
#define TAR_FILE_MODE  2
#define ZOO_FILE_MODE  3
#define ZIP_FILE_MODE  4
#define LHA_FILE_MODE  5
#define ARC_FILE_MODE  6

#define NO_COMPRESS                 0
#define FREEZE_COMPRESS             1
#define MULTIPLE_FREEZE_COMPRESS    2
#define COMPRESS_COMPRESS           3
#define MULTIPLE_COMPRESS_COMPRESS  4
#define GZIP_COMPRESS               5
#define MULTIPLE_GZIP_COMPRESS      6
#define ZOO_COMPRESS                7
#define LHA_COMPRESS                8
#define ARC_COMPRESS                9
#define ZIP_COMPRESS                10


#define COMPRESS_EXTENSIONS  {                                           \
			       { ".F",   FREEZE_COMPRESS },              \
			       { ".TFR", FREEZE_COMPRESS },              \
			       { ".Faa", MULTIPLE_FREEZE_COMPRESS },     \
			       { ".Z",   COMPRESS_COMPRESS },            \
			       { ".TZ",  COMPRESS_COMPRESS },            \
			       { ".TZR", COMPRESS_COMPRESS },            \
			       { ".Xaa", MULTIPLE_COMPRESS_COMPRESS },   \
			       { ".z",   GZIP_COMPRESS },                \
			       { ".gz",  GZIP_COMPRESS },                \
			       { ".tz",  GZIP_COMPRESS },                \
			       { ".tzr", GZIP_COMPRESS },                \
			       { ".tgz", GZIP_COMPRESS },                \
			       { ".TGZ", GZIP_COMPRESS },                \
			       { ".taz", GZIP_COMPRESS },                \
			       { ".TAZ", GZIP_COMPRESS },                \
			       { ".tpz", GZIP_COMPRESS },                \
			       { ".TPZ", GZIP_COMPRESS },                \
			       { ".xaa", MULTIPLE_GZIP_COMPRESS },       \
			       { ".zoo", ZOO_COMPRESS },                 \
			       { ".ZOO", ZOO_COMPRESS },                 \
			       { ".lzh", LHA_COMPRESS },                 \
			       { ".LZH", LHA_COMPRESS },                 \
			       { ".arc", ARC_COMPRESS },                 \
			       { ".ARC", ARC_COMPRESS },                 \
			       { ".zip", ZIP_COMPRESS },                 \
			       { ".ZIP", ZIP_COMPRESS }                  \
			     }



#define SORT_BY_NAME       1
#define SORT_BY_MOD_TIME   2
#define SORT_BY_CHG_TIME   3
#define SORT_BY_ACC_TIME   4
#define SORT_BY_SIZE       5
#define SORT_BY_OWNER      6
#define SORT_BY_GROUP      7

#define DEFAULT_FILE_SPEC "*"

#define FILE_SEPARATOR_CHAR   '/'
#define FILE_SEPARATOR_STRING "/"

#define BOOL       unsigned char
#define CR         13
#define LF         10
#define ESC        27
#define LOGIN_ESC  '.'

#define MIN_WIDTH  41
#define MIN_HEIGHT 24

#define DIR_WINDOW_X       1  
#define DIR_WINDOW_Y       2 
#define DIR_WINDOW_WIDTH   (COLS - 23)
#define DIR_WINDOW_HEIGHT  14

#define FILE_WINDOW_1_X      1
#define FILE_WINDOW_1_Y      17
#define FILE_WINDOW_1_WIDTH  (COLS - 23)
#define FILE_WINDOW_1_HEIGHT (LINES - 20 )

#define FILE_WINDOW_2_X      1
#define FILE_WINDOW_2_Y      2
#define FILE_WINDOW_2_WIDTH  (COLS - 23)
#define FILE_WINDOW_2_HEIGHT (LINES - 5) 

#define ERROR_WINDOW_WIDTH   40
#define ERROR_WINDOW_HEIGHT  10
#define ERROR_WINDOW_X       ((COLS - ERROR_WINDOW_WIDTH) >> 1)  
#define ERROR_WINDOW_Y       ((LINES - ERROR_WINDOW_HEIGHT) >> 1)

#define PATH_LENGTH          1024
#define FILE_SPEC_LENGTH     (12 + 1)
#define DISK_NAME_LENGTH     (12 + 1)
#define LL_LINE_LENGTH       512
#define TAR_LINE_LENGTH      512
#define ZOO_LINE_LENGTH      512
#define ZIP_LINE_LENGTH      512
#define LHA_LINE_LENGTH      512
#define ARC_LINE_LENGTH      512
#define MESSAGE_LENGTH       (PATH_LENGTH + 80 + 1)
#define COMMAND_LINE_LENGTH  4096
#define MODE_1               0
#define MODE_2               1
#define MODE_3               2
#define MODE_4               3


#ifdef __NeXT__
#include <sys/ttydev.h>
#define QUICK_BAUD_RATE      B9600
#else
#define QUICK_BAUD_RATE      9600
#endif /* __NeXT */

#define ESCAPE               goto FNC_XIT

#define BLKSIZ               512  /* Blockgroesse */



typedef struct _file_entry
{
  struct _file_entry *next;
  struct _file_entry *prev;
  struct _dir_entry  *dir_entry;
  struct stat        stat_struct;
  BOOL               tagged;
  BOOL               matching;
  char               name[1];
/*  char               symlink_name[]; */  /* Folgt direkt dem Namen, falls */
					/* Eintrag == symbolischer Link  */
} FileEntry;



typedef struct _dir_entry
{
  struct _file_entry *file;
  struct _dir_entry  *next;
  struct _dir_entry  *prev;
  struct _dir_entry  *sub_tree;
  struct _dir_entry  *up_tree;
  unsigned long      total_bytes;
  unsigned long      matching_bytes;
  unsigned long      tagged_bytes;
  unsigned int       total_files;
  unsigned int       matching_files;
  unsigned int       tagged_files;
  int                cursor_pos;
  int                start_file;
  struct   stat      stat_struct;
  BOOL               access_denied;
  BOOL               global_flag;
  BOOL               login_flag;
  char               name[1];
} DirEntry;



typedef struct
{
  unsigned long      indent;
  DirEntry           *dir_entry;
  unsigned short     level;
} DirEntryList;


typedef struct
{
  FileEntry          *file;
} FileEntryList;


typedef struct 
{
  char          login_path[PATH_LENGTH + 1];
  char          path[PATH_LENGTH + 1];
  char          file_spec[FILE_SPEC_LENGTH + 1];
  char          disk_name[DISK_NAME_LENGTH + 1];
  int           kind_of_sort;
  unsigned long disk_space;
  unsigned long disk_total_files;
  unsigned long disk_total_bytes;
  unsigned long disk_matching_files;
  unsigned long disk_matching_bytes;
  unsigned long disk_tagged_files;
  unsigned long disk_tagged_bytes;
  unsigned int  disk_total_directories;
  int           disp_begin_pos;
  int           cursor_pos;
  DirEntry      *tree;
} Statistic;


typedef union
{
  struct
  {
    char      new_modus[11];
  } change_modus;

  struct
  {
    unsigned  new_owner_id;
  } change_owner;

  struct
  {
    unsigned  new_group_id;
  } change_group;

  struct
  {
    char      *command;
  } execute;

  struct
  {
    Statistic *statistic_ptr;
    DirEntry  *dest_dir_entry;
    char      *to_file;
    char      *to_path;
    BOOL      confirm;
  } copy;

  struct
  {
    char      *new_name;
    BOOL      confirm;
  } rename;

  struct
  {
    DirEntry  *dest_dir_entry;
    char      *to_file;
    char      *to_path;
    BOOL      confirm;
  } mv;

  struct
  {
    FILE      *pipe_file;
  } pipe_cmd;

} FunctionData;


typedef struct
{
  FileEntry     *new_fe_ptr;
  FunctionData  function_data;
} WalkingPackage;



extern WINDOW *dir_window;
extern WINDOW *small_file_window;
extern WINDOW *big_file_window;
extern WINDOW *file_window;
extern WINDOW *error_window;

extern Statistic statistic;
extern Statistic disk_statistic;
extern int       mode;
extern char      message[];

extern char *getenv(const char *);

extern void DisplayMenu(void);
extern void DisplayDiskStatistic(void);
extern void DisplayDirStatistic(DirEntry *dir_entry);
extern void DisplayDirParameter(DirEntry *dir_entry);
extern void DisplayDirTagged(DirEntry *dir_entry);
extern void DisplayDiskTagged(void);
extern void DisplayFileParameter(FileEntry *file_entry);
extern void DisplayGlobalFileParameter(FileEntry *file_entry);
extern int  ReadTree(DirEntry *dir_entry, char *path);
extern int  ReadTreeFromTAR(DirEntry *dir_entry, FILE *f);
extern int  ReadTreeFromZOO(DirEntry *dir_entry, FILE *f);
extern int  ReadTreeFromZIP(DirEntry *dir_entry, FILE *f);
extern int  ReadTreeFromLHA(DirEntry *dir_entry, FILE *f);
extern int  ReadTreeFromARC(DirEntry *dir_entry, FILE *f);
extern int  GetDiskParameter(char *path, char *volume_name, long unsigned int *avail_bytes);
extern int  HandleDirWindow(DirEntry *start_dir_entry,
			    int disp_begin_pos,
			    int cursor_pos
			   );
extern void DisplayFileWindow(DirEntry *dir_entry, int start_file_no, int hilight_no);
extern int  Init(void);
extern char *GetPath(DirEntry *dir_entry, char *buffer);
extern BOOL Match(char *file_name);
extern int  SetMatchSpec(char *new_spec);
extern int  SetFileSpec(char *file_spec);
extern void Error(char *msg, char *module, int line);
extern void Warning(char *msg);
extern void SetFileMode(int new_file_mode);
extern int  HandleFileWindow(DirEntry *dir_entry, 
			     int start_file,
			     int cursor_pos,
			     BOOL global_flag);
extern char *GetAttributes(ushort modus, char *buffer);
extern void SwitchToSmallFileWindow(void);
extern void SwitchToBigFileWindow(void);
extern int  ReadGroupEntries(void);
extern char *GetGroupName(unsigned int gid);
extern int  GetGroupId(char *name);
extern int  ReadPasswdEntries(void);
extern char *GetPasswdName(unsigned int uid);
extern int  GetPasswdUid(char *name);
extern char *GetFileNamePath(FileEntry *file_entry, char *buffer);
extern char *GetRealFileNamePath(FileEntry *file_entry, char *buffer);
extern int  SystemCall(char *command_line);
extern int  QuerySystemCall(char *command_line);
extern int  SilentSystemCall(char *command_line);
extern int  View(char *file_path);
extern int  ViewHex(char *file_path);
extern int  Edit(char *file_path);
extern void DisplayAvailBytes(void);
extern void DisplayFileSpec(void);
extern void Quit(void);
extern void ReadFileSpec(void);
extern int  InputString(char *s, int y, int x, int length, char *term);
extern void RotateFileMode(void);
extern int  Execute(DirEntry *dir_entry, FileEntry *file_entry);
extern int  Pipe(DirEntry *dir_entry, FileEntry *file_entry);
extern int  PipeTaggedFiles(FileEntry *fe_ptr, WalkingPackage *walking_package);
extern int  GetPipeCommand(char *pipe_command);
extern void GetKindOfSort(void);
extern void SetKindOfSort(int new_kind_of_sort);
extern int  ChangeFileModus(FileEntry *fe_ptr);
extern int  ChangeDirModus(DirEntry *de_ptr);
extern int  GetNewFileModus(int y, int x, char *modus, char *term);
extern int  GetModus(char *modus);
extern int  SetFileModus(FileEntry *fe_ptr, WalkingPackage *walking_package);
extern int  CopyTaggedFiles(FileEntry *fe_ptr, WalkingPackage *walking_package);
extern int  CopyFile(Statistic *statistic_ptr, FileEntry *fe_ptr, unsigned char confirm, char *to_file, DirEntry *dest_dir_entry, char *to_dir_path);
extern int  MoveTaggedFiles(FileEntry *fe_ptr, WalkingPackage *walking_package);
extern int  MoveFile(FileEntry *fe_ptr, unsigned char confirm, char *to_file, DirEntry *dest_dir_entry, char *to_dir_path, FileEntry **new_fe_ptr);
extern int  InputChoise(char *msg, char *term);
extern void Message(char *msg);
extern int  GetDirEntry(DirEntry *tree, DirEntry *current_dir_entry, char *dir_path, DirEntry **dir_entry, char *to_path);
extern int  GetFileEntry(DirEntry *de_ptr, char *file_name, FileEntry **file_entry);
extern int  GetCopyParameter(char *from_file, char *to_file, char *to_dir);
extern int  GetMoveParameter(char *from_file, char *to_file, char *to_dir);
extern int  ChangeFileOwner(FileEntry *fe_ptr);
extern int  GetNewOwner(int st_uid);
extern int  SetFileOwner(FileEntry *fe_ptr, WalkingPackage *walking_package);
extern int  ChangeDirOwner(DirEntry *de_ptr);
extern int  ChangeFileGroup(FileEntry *fe_ptr);
extern int  GetNewGroup(int st_gid);
extern int  SetFileGroup(FileEntry *fe_ptr, WalkingPackage *walking_package);
extern int  ChangeDirGroup(DirEntry *de_ptr);
extern void DisplayDirHelp(void);
extern void DisplayFileHelp(void);
extern void ClearHelp(void);
extern int  GetAvailBytes(long unsigned int *avail_bytes);
extern int  DeleteDirectory(DirEntry *dir_entry);
extern int  ExecuteCommand(FileEntry *fe_ptr, WalkingPackage *walking_package);
extern int  GetCommandLine(char *command_line);
extern void DisplayVersion(void);
extern int  DeleteFile(FileEntry *fe_ptr);
extern int  RemoveFile(FileEntry *fe_ptr);
extern int  RenameDirectory(DirEntry *de_ptr, char *new_name);
extern int  RenameFile(FileEntry *fe_ptr, char *new_name, FileEntry **new_fe_ptr);
extern int  RenameTaggedFiles(FileEntry *fe_ptr, WalkingPackage *walking_package);
extern int  GetRenameParameter(char *old_name, char *new_name);
extern char *CTime(time_t f_time, char *buffer);
extern int  LoginDisk(char *path);
extern int  GetNewLoginPath(char *path);
extern void PrintSpecialString(WINDOW *win, int y, int x, char *str);
extern char *FormFilename(char *dest, char *src, unsigned int max_len);
extern char *CutFilename(char *dest, char *src, unsigned int max_len);
extern void   Fnsplit(char *path, char *dir, char *name);
extern void MakeExtractCommandLine(char *command_line, char *path, char *file, char *cmd);
extern int MakeDirectory(DirEntry *father_dir_entry);
extern time_t Mktime(struct tm *tm);
extern int TryInsertArchiveDirEntry(DirEntry *tree, char *dir, struct stat *stat);
extern int InsertArchiveFileEntry(DirEntry *tree, char *path, struct stat *stat);
extern int MinimizeArchiveTree(DirEntry *tree);
extern void HitReturnToContinue(void);
extern int  TermcapWgetch(WINDOW *win);
extern void TermcapVidattr(int attr );
extern void TermcapInitscr(void);
extern void TermcapEndwin(void);
extern int  BuildFilename( char *in_filename, char *pattern, char *out_filename);
extern int  ViKey( int ch );
extern int  GetCompressMethod( char *filename );
